#!/bin/bash
# NAME            applyLSIPatch.sh
# SUMMARY         Script for applying LSI patchfile to u-boot
# VERSION         %version: 1 %
#
#        Copyright 2012 NetApp, Inc. All Rights Reserved.
#
# DESCRIPTION:
# This script is used to apply an LSI patch to a vanilla u-boot source tree
# The entire process is not automated yet, but these steps can get things
# to the point where you can compare the LSI code to the current u-boot
# code in CM Synergy.
# Further automation could probably be done, but for now we'll leave useful
# steps as comments and let things progress incrementally as further LSI
# drops are provided and as more NetApp modifications are added.

if [ `uname` != "Linux" ] ; then
  echo "This can only be run on a Linux machine."
  exit 1
fi

if [ $# -eq "0" ] ; then
  echo "Usage: `basename $0` <patchfile>"
  exit 1
fi

if [ ! -e $1 ] ; then
  echo "Error: $1 does not exist"
  exit 1
fi

# If this next check fails, the user probably called this script from outside
# the expected directory.
if [ ! -e "./u%%boot%%2010.12.tar.bz2" ] ; then
  echo "u-boot bz2 file not found - This must be run from the patchfiles dir"
  exit 1
fi

# Remove working dir if someone has already run this script.
rm -rf u%%boot%%2010.12

# Extract stripped archive to "u-boot-2010.12"
# On Linux: tar xjf u%%boot%%2010.12.tar.bz2
# Generic for Solaris & Linux:
bzip2 --stdout --decompress --keep u%%boot%%2010.12.tar.bz2 | tar xf -

# Go into dir
cd u-boot-2010.12

# Apply patchfile ($1)
patch -p1 -i ../$1

# Leave dir
cd ..

# Call the rename script to change "-" to "%%" for comparing to CM Synergy
./renameForCM.sh

# Look for differences from current files
echo "***Comparing to version in change control***"
diff -qr u%%boot%%2010.12 ../u%%boot%%2010.12

# The following command can be used to get a list of changed files (but
# will not work if files were added).
#   diff -qr u%%boot%%2010.12 ../u%%boot%%2010.12 | awk '{print $2}'

# From here it's a manual process of comparing files and getting them into CM
# Synergy. If it's a handful of files, it's probably easiest to manually
# check them out from the GUI, then you can just 'cp' the newer files over
# the older ones. Then check the history view to make sure the history links
# are appropriate.

# OTHER NOTES:
# If there are many files, you could use the ccm CLI to try and speed things
# up. You have to set up a working ccm CLI session first, which is outside the
# scope of this README (search the wiki for 'ccm cli'). Here are some commands
# that were used to check out many files that had changed for the initial LSI
# u-boot drop:
#   diff -qr u%%boot%%2010.12 ../u%%boot%%2010.12 | awk '{print $2}' > files.txt
#   cat files.txt | xargs ccm co -task WIC~### -t "00.250.xxx.00"
# Then 'cp' each file from u%%boot%%2010.12 to ../u%%boot%%2010.12
#
# If there are new files, there is nothing to check out, so you can use the CLI
# to make that a bit easier. In this case, manually 'cp' each file into the
# CM Synergy-managed u%%boot%%2010.12 tree. If you copy the file first, then
# CM Synergy will use the contents when creating the new file object. Otherwise
# it will create a blank template file and you will need to 'cp' the actual
# file on top of it. To create one or more files from the CLI you can use:
#   ccm create -type <csrc|incl|makefile|ascii|binary|asmsrc|dir> -task WIC~### -v "00.250.xxx.00" path/to/file1 [path/to/file2] [path/to/fileN...]
# If you build a list of filenames, you can use the "cat files | xargs"
# method above to create a bunch at once. You just need to separate them by
# type (csrc can be done in a single batch, then incl, then ...).

