/*******************************************************************************

NAME            i2c.c
VERSION         %version: 5 %
UPDATE DATE     %date_modified: Thu Oct 17 11:30:01 2013 %
PROGRAMMER      %created_by:    weberc %

        Copyright 2013 NetApp, Inc. All rights reserved.

*******************************************************************************/
/***************************************************************************
 *                                                                         *
 *  Copyright(C) 2012 LSI Corporation.  		                           *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  i2c.c
 *          Title:  File containing functions for transactions in I2C channel
 *     Programmer:  Naresh Madhusudana
 *  Creation Date:  Aug 15, 2011
 *
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   --------------------------------------------------------
 *  15/08/11  NM   This file has the I2C interface Init, Read, Write functions
 *  
 *
 * Description
 * ------------
 *
 *    This file consists of functions that are used to perform read and write
 *    from an I2C channel. This is used only for readign the SPD of DDR3 DIMM.
 *
 *-------------------------------------------------------------------------
 */

#include <common.h>
#include "include/lsisas3108_i2c.h"
#include <asm/io.h>
#include <asm/processor.h>
#include <asm/ppc4xx.h>


DECLARE_GLOBAL_DATA_PTR;

/*
 * Name: _i2c_bus_reset(void)
 *
 * Description: This fucntion is used to perform I2C bus reset 
 *
 * Arguments : None
 *
 * Return value : None
 */
static void _i2c_bus_reset(void)
{
	int cnt=0;

	while ( (in_le16((u16*)IIC_MONITOR ) & 0x0002) == 0 )
	{
		/************************************************/
		/* A slave device is holding the data line low. */
		/* Try toggling the clock line to get the slave */
		/* to release the data line.  If it doesn't     */
		/* work after 20 clocks, give up.               */
		/************************************************/
		if (cnt > 20)
			break;

		out_le16((u16*)IIC_MONITOR, (in_le16((u16*)IIC_MONITOR)) | 0x0004);
		udelay(5);
		out_le16((u16*)IIC_MONITOR, (in_le16((u16*)IIC_MONITOR)) & ~0x0004);
		udelay(5);
		cnt++;
	}

	if ( (in_le16((u16*)IIC_MONITOR ) & 0x0002) == 0 )
	{
		printf("i2c data bus stuck low\n");
	}

	/* clear the interrupt status MI and SI */
	/* write 1 in SI and MI to clear these fields */
	out_le16((u16*)IIC_INTSTS, IIC_INTSTS_SI_CLR | IIC_INTSTS_MI_CLR);

	/* Clear/Flush the Master Tx FIFO */
	out_le16((u16*)IIC_MCMD,IIC_MCMD_FLUSH_TX_FIFO | IIC_MCMD_SEND_CMD);

	/* Reset the i2c core - assures that I2c bus is free */
	out_le16((u16*)IIC_SOFTRST,IIC_SOFTRST_SRST);

}

/*
 * Name: i2c_init(speed,slaveaddress)
 *
 * Description: This fucntion is used to initialize the I2C interface for a particular slave
 *
 * Arguments : speed: speed of operation 
 * 	       slaveadd: I2C slave address
 *
 * Return value : None
 */
void i2c_init(int speed, int slaveadd)
{
	/* We need only SPD I2C in u-boot*/
	_i2c_bus_reset();

	/* Enable master and slave bit */
	out_le16((u16*)IIC_GLBCTRL,IIC_GLBCTRL_IE | IIC_GLBCTRL_SE | IIC_GLBCTRL_ME);
	/* Enable master and slave interrupts */
	out_le16((u16*)IIC_INTEN,IIC_INTEN_MIE | IIC_INTEN_SIE);
	/* Enable individual master interrupt */
	out_le16((u16*)IIC_MINTEN,IIC_MINTEN_ALL);
	/* Enable individual slave interrupt */
	out_le16((u16*)IIC_SINTEN,IIC_SINTEN_ALL);
	/* Configure clock setup register */
	/* PCLK = 200MHz and I2C = Standard Mode (100KHz) */
	/* SCL High time = 10 us -1 PCLK = 1000 -1 = 0x3E7 */
	out_le16((u16*)IIC_SCLHIGH,IIC_SCLHIGH_TIME);
	/* SCL low time = 10 us -1 PCLK = 0x3E7 */
	out_le16((u16*)IIC_SCLLOW,IIC_SCLLOW_TIME);
	/* SDA setup = 1.3 us with margin */
	out_le16((u16*)IIC_SDASETUPT,IIC_SDASETUPT_TIME);
	/* SDA hold = 700 ns with margin */
	out_le16((u16*)IIC_SDAHOLDT,IIC_SDAHOLDT_TIME);
	/*   spikes filter width 35ns */
	out_le16((u16*)IIC_SPFILTLEN,IIC_SPFILT_LENGTH);
	/* configure time out control register */
	out_le16((u16*)IIC_TCLKDIV,IIC_TCLKDIV_DS);
	/* Desired wait time out = 20 ms*/
	out_le16((u16*)IIC_WTCTRL,ICC_TIMER_TE | IIC_WTCTRL_TLV);
	/* Enable each IBML timer */
	/* Desired IBML time out = 25 ms*/
	out_le16((u16*)IIC_IBMLTOUT,ICC_TIMER_TE | IIC_IBMLTOUT_ITLV);
	/* Desired IBML ME time out = 10 ms*/
	out_le16((u16*)IIC_IBMLLMEXT,ICC_TIMER_TE | IIC_IBMLLMEXT_IMTLV);
	/* Desired IBML SE time out = 25 ms*/
	out_le16((u16*)IIC_IBMLLSEXT,ICC_TIMER_TE | IIC_IBMLLSEXT_ISTLV);

}


/*
 * Name: i2c_transfer(command, chip, address, addr_len, data, data_len)
 *
 * Description: This fucntion is used to perform I2C read/Write operations
 *
 * Arguments : cmd_type : specifies if the command is read or write
 * 	       chip     : I2C slave
 * 	       addr[]   : memory address
 * 	       addr_len : address length in bytes(2/3 bytes)
 * 	       data[]   : buffer for data
 * 	       data_len : data length in bytes
 *
 *
 * Return value : None
 */
static int i2c_transfer(unsigned char cmd_type,
			unsigned char chip,
			unsigned char addr[],
			unsigned char addr_len,
			unsigned char data[],
			unsigned short data_len)
{
	unsigned char* ptr;
	int            reading;
	int            tran, cnt;
	int            result;
	int            byte_count, byte_xfer;
	unsigned short mIntSts;
	int            i;
	int            attempts;

	if (data == 0 || data_len == 0) {
		/* Don't support data transfer of no length or to address 0 */
		printf( "i2c_transfer: bad call\n" );
		return IIC_NOK;
	}

	/* Try up to three times - if the bus is held low by a slave device, the
	 * first transfer will fail, but the bus will be cleaned up by the attempt
	 * and the second transfer should succeed. Other conditions may exist where
	 * an automatic retry or two may be desired rather than having the calling
	 * code be forced to retry a number of times. */
	for (attempts=0; attempts<3; attempts++)
	{
		if (addr && addr_len) {
			ptr = addr;
			cnt = addr_len;
			reading = 0;
		} else {
			ptr = data;
			cnt = data_len;
			reading = cmd_type;
		}


		/* flush the Master/Slave Databuffers */
		out_le16((u16*)IIC_MCMD, IIC_MCMD_FLUSH_TX_FIFO | IIC_MCMD_SEND_CMD);

		/* 7-bit adressing */
		out_le16((u16*)IIC_MADDR2, 0);
		out_le16((u16*)IIC_MADDR1, chip);

		/* Clear all Master interrupts */
		out_le16((u16*)IIC_INTSTS,IIC_INTSTS_MI_CLR);

		/* In some cases, the RX buffer was not empty for some as-yet unknown
		 * reason. When that happens, just read from the buffer until it is
		 * empty. Added a print statement so we know if/when this happens. */
		while( in_le16((u16*)IIC_MRXFIFOS) & IIC_MRXFIFOS_FD )
		{
			udelay(1);
			unsigned char data = (unsigned char)in_le16((u16*)IIC_MDATA);
			printf("i2c discard: %02x\n", data);
		}

		tran   = 0;
		result = IIC_OK;

		while(tran != cnt && (result == IIC_OK))
		{
			byte_count = (cnt - tran) > IIC_MAX_BYTE_XFER ? \
								IIC_MAX_BYTE_XFER : cnt - tran;

			if (reading)
			{
				out_le16((u16 *)IIC_MRXLEN,byte_count);
				out_le16((u16*)IIC_MADDR1,
						((in_le16((u16*)IIC_MADDR1))|IIC_MADDR1_RD));
			}
			else
			{
				out_le16((u16*)IIC_MADDR1,
						((in_le16((u16*)IIC_MADDR1))& IIC_MADDR1_WR));
				out_le16((u16 *)IIC_MTXLEN,byte_count);
				/* Fill TX FIFO from data buffer */
				for(byte_xfer=0; byte_xfer < byte_count; byte_xfer++)
				{
					out_le16((u16*)IIC_MDATA, ptr[tran+byte_xfer]);
				}

			}
			out_le16((u16 *)IIC_MCMD,IIC_MCMD_AUTO_MODE | IIC_MCMD_SEND_CMD);

			/* Transfer is in progress
			 * we have to wait for upto 5 bytes of data
			 * 1 byte chip address+r/w bit then byte_count bytes
			 * of data.
			 * udelay(10) is 1 bit time at 100khz
			 * Doubled for slop. 20 is too small. 
			 * NOTE: Above comments are the original comments for this loop.
			 *   For debug purposes, we've extended the time by a factor of 10
			 *   just to be on the safe side. Instrumenting the code shows that
			 *   normal reads take up to 61 delays. The only I2C usage in the
			 *   bootloader is reading the DIMM SPD data, so it doesn't hurt
			 *   much to wait longer, considering that if these transfers
			 *   don't work, the system won't boot.
			 */
			i = 2*5*8*10;
			do
			{
				/* Get status */
				mIntSts = in_le16((u16*)IIC_MINTS);
				udelay(10);
				i--;
			} while (!(mIntSts & IIC_MINTS_TS) && (i > 0));


			if((mIntSts & IIC_MINTS_SS) != IIC_MINTS_SS)
			{
				printf( "i2c_transfer: IIC_NOK_ICT\n" );
				result = IIC_NOK_ICT;
			}

			switch(mIntSts & IIC_MINTS_TR_FAILED){
			case IIC_MINTS_TOUT:
				printf( "i2c_transfer: IIC_NOK_TOUT\n" );
					result = IIC_NOK_TOUT;
					break;
			case IIC_MINTS_AL:
				printf( "i2c_transfer: IIC_NOK_LA\n" );
					result = IIC_NOK_LA;
					break;
			case IIC_MINTS_NAK_DATA:
			case IIC_MINTS_NAK_ADDR:
				printf( "i2c_transfer: IIC_NOK_XFRA\n" );
					result = IIC_NOK_XFRA;
					break;
			default: break;
			}


			if ((reading) && (result == IIC_OK))
			{
				/* Are there data in buffer */
				if( in_le16((u16*)IIC_MRXFIFOS) & IIC_MRXFIFOS_FD )
				{
					udelay(1);
					for (byte_xfer=0; byte_xfer<byte_count; byte_xfer++)
					{
						ptr[tran+byte_xfer] = (unsigned char)in_le16((u16*)IIC_MDATA);
					}
				}
				else
				{
					printf( "i2c_transfer: IIC_NOK_DATA\n" );
					result = IIC_NOK_DATA;		/*	"No data in RX FIFO" */
				}
			}

			tran += byte_count;
			if ((ptr == addr)&&(tran == cnt))
			{
				ptr = data;
				cnt = data_len;
				tran = 0;
				reading = cmd_type;
			}
		}

		if (result == IIC_OK)
		{
			/* Command was successful, we're done */
			break;
		}
		else
		{
			/* Retry unsuccessful commands - Reset everything and then delay
			 * a bit to let things settle out back to a good state. */
			printf( "i2c_transfer: retrying attempt %d\n", attempts );
			i2c_init(0,0);
			udelay(100);
		}
	}

	return (result);
}

/*
 * Name: i2c_probe(chip)
 *
 * Description: This fucntion is used to check if device is present on I2C bus 
 *
 * Arguments : chip: I2C slave address
 *
 * Return value : None
 */
int i2c_probe(uchar chip)
{
	uchar buf[1];

	buf[0] = 0;

	/*
	 * What is needed is to send the chip address and verify that the
	 * address was <ACK>ed (i.e. there was a chip at that address which
	 * drove the data line low).
	 */
	return (i2c_transfer(1, chip << 1, 0,0, buf, 1) != 0);
}

/*
 * Name: i2c_read(chip, addr,addr_len, buffer, len)
 *
 * Description: This fucntion is used to perform I2C read and is calls i2c_transfer
 *
 * Arguments : chip: Slave address
 * 	       addr: memory address
 * 	       addr_len: address length in bytes
 * 	       buffer: buffer to place read data
 * 	       len: data length in bytes
 *
 *
 * Return value : None
 */

int i2c_read(uchar chip, uint addr, int alen, uchar * buffer, int len)
{
	uchar xaddr[4];
	int ret;

	if (alen > 4) {
		printf ("I2C read: addr len %d not supported\n", alen);
		return 1;
	}

	if (alen > 0) {
		xaddr[0] = (addr >> 24) & 0xFF;
		xaddr[1] = (addr >> 16) & 0xFF;
		xaddr[2] = (addr >> 8) & 0xFF;
		xaddr[3] = addr & 0xFF;
	}

	if ((ret = i2c_transfer(1, chip<<1, &xaddr[4-alen], alen, buffer, len)) != 0) {
		if (gd->have_console)
			printf( "I2c read: failed %d\n", ret);
		return 1;
	}
	return 0;
}

/*
 * Name: i2c_write(chip, addr,addr_len, buffer, len)
 *
 * Description: This fucntion is used to perform I2C write and calls i2c_transfer
 *
 * Arguments : chip: Slave address
 * 	       addr: memory address
 * 	       addr_len: address length in bytes
 * 	       buffer: buffer holdign the write data
 * 	       len: data length in bytes
 *
 * Return value : None
 *
 */
int i2c_write(uchar chip, uint addr, int alen, uchar * buffer, int len)
{
	uchar xaddr[4];

	if (alen > 4) {
		printf ("I2C write: addr len %d not supported\n", alen);
		return 1;

	}

	if (alen > 0) {
		xaddr[0] = (addr >> 24) & 0xFF;
		xaddr[1] = (addr >> 16) & 0xFF;
		xaddr[2] = (addr >> 8) & 0xFF;
		xaddr[3] = addr & 0xFF;
	}

	return (i2c_transfer(0, chip<<1, &xaddr[4-alen], alen, buffer, len ) != 0);
}
