/***************************************************************************
 *                                                                         *
 *  Copyright 2011-12 LSI Corporation.  All rights reserved.               *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corp.  The         *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  mpic.h
 *          Title:  MPIC Hardware Header
 *     Programmer:  Mike Fry
 *  Creation Date:  October 27, 2009
 *
 *
 * Description
 * ------------
 * Hardware description headers for the MPIC Interrupt Controller
 *
 *-------------------------------------------------------------------------
 */

#ifndef __MPIC_H_INCLUDED
#define __MPIC_H_INCLUDED

/***********************************************************************/
/* MPIC Vector / Priority Register (MPICx_VP0-MPICx_VP127)             */
/***********************************************************************/
/*
 * There are 128 vector/priority registers.  The DCR address offset within the
 * MPIC DCR address block for register n is calculated as follows:
 *
 *     MPIC_VECTOR_PRIORITY_BASE_DCRA + (n * MPIC_VECTOR_PRIORITY_INCR)
 */
#define SASHW_DCR_MPIC                  (0xF0000000)  // Multi-processor interrupt controller
#define MPIC_VECTOR_PRIORITY_BASE_DCRA  (SASHW_DCR_MPIC + 0xFC10000)
#define MPIC_VECTOR_PRIORITY_INCR       (0x20)

/* Macro to give the Vector Priority Register DCR Address */
#define MPIC_VECTOR_PRIORITY_DCRA(n)    (MPIC_VECTOR_PRIORITY_BASE_DCRA + ((n) * MPIC_VECTOR_PRIORITY_INCR))

#define MPIC_VECTOR_PRIORITY_VEC_MASK   (0x000000FF)   // Interrupt vector mask
#define MPIC_VECTOR_PRIORITY_VEC_SHIFT  (0)            // Interrupt vector shift
#define MPIC_VECTOR_PRIORITY_PRI_MASK   (0x000F0000)   // Interrupt priority mask
#define MPIC_VECTOR_PRIORITY_PRI_SHIFT  (16)           // Interrupt priority shift
#define MPIC_VECTOR_PRIORITY_SENSE      (0x00400000)   // Sense: 0 = edge, 1 = level
#define MPIC_VECTOR_PRIORITY_POLARITY   (0x00800000)   // Polarity: 0 = active low or negative edge
                                                       //           1 = active high or positive edge
#define MPIC_VECTOR_PRIORITY_ACT        (0x40000000)   // Activity: 0 = Okay to modify, 1 = in use
#define MPIC_VECTOR_PRIORITY_MASK       (0x80000000)   // Mask: 0 = enabled, 1 = disabled

#if  !defined(__LANGUAGE_ASM__)

typedef struct _MPIC_VECTOR_PRIORITY
{
    union
    {
        struct
        {
            #if defined( __LITTLE_ENDIAN__ ) || defined( __LITTLE_ENDIAN )
                U32 Vector          :   8;  /*  0-7  Bits 24-31 Interrupt vector   */
                U32 Reserved16to23  :   8;  /*  8-15 Bits 16-23                    */
                U32 Priority        :   4;  /* 16-19 Bits 12-15 Interrupt priority */
                U32 Reserved10to11  :   2;  /* 20-21 Bits 10-11                    */
                U32 Sense           :   1;  /* 22    Bits 9     Sense              */
                U32 Polarity        :   1;  /* 23    Bits 8     Polarity           */
                U32 Reserved2to7    :   6;  /* 24-29 Bits 2-7                      */
                U32 Activity        :   1;  /* 30    Bits 1     Activity           */
                U32 Mask            :   1;  /* 31    Bits 0     Mask               */
            #else
                U32 Mask            :   1;  /* 31    Bits 0     Mask               */
                U32 Activity        :   1;  /* 30    Bits 1     Activity           */
                U32 Reserved2to7    :   6;  /* 24-29 Bits 2-7                      */
                U32 Polarity        :   1;  /* 23    Bits 8     Polarity           */
                U32 Sense           :   1;  /* 22    Bits 9     Sense              */
                U32 Reserved10to11  :   2;  /* 20-21 Bits 10-11                    */
                U32 Priority        :   4;  /* 16-19 Bits 12-15 Interrupt priority */
                U32 Reserved16to23  :   8;  /*  8-15 Bits 16-23                    */
                U32 Vector          :   8;  /*  0-7  Bits 24-31 Interrupt vector   */
            #endif
        }   Bits;
        U32 Word;
    } u;
} MPIC_VECTOR_PRIORITY, *PTR_MPIC_VECTOR_PRIORITY;

#endif

/***********************************************************************/
/* MPIC Destination Register (MPICx_DST0-MPICx_DST127)                 */
/***********************************************************************/
/*
 * There are 128 destination registers.  The DCR address offset within the
 * MPIC DCR address block for register n is calculated as follows:
 *
 *     MPIC_DESTINATION_BASE_DCRA + (n * MPIC_VECTOR_PRIORITY_INCR)
 *
 * NOTE: These registers use the Processor Select bit and structure
 *       ( _MPIC_PROCESSOR_SELECT ) definitions below.
 */
#define MPIC_DST_BASE_DCRA      (SASHW_DCR_MPIC + 0xFC10010)
#define MPIC_DST_INCR           (0x20)

#define MPIC_DST_DCRA(n)        (MPIC_DST_BASE_DCRA + ((n) * MPIC_DST_INCR) )

/********************************************************************************/
/* MPIC Inter Processor Interrupt Dispatch Registers (MPICx_IPID0-MPICx_IPID3)  */
/********************************************************************************/
/*
 * NOTE: These registers use the Processor Select bit and structure
 *       ( _MPIC_PROCESSOR_SELECT ) definitions below.
 */
#define MPIC_IPID0_PR_DCRA      (SASHW_DCR_MPIC + 0x0FC00040)
#define MPIC_IPID0_P0_DCRA      (SASHW_DCR_MPIC + 0x0FC20040)
#define MPIC_IPID0_P1_DCRA      (SASHW_DCR_MPIC + 0x0FC21040)
#define MPIC_IPID0_P2_DCRA      (SASHW_DCR_MPIC + 0x0FC22040)
#define MPIC_IPID0_P3_DCRA      (SASHW_DCR_MPIC + 0x0FC23040)

#define MPIC_IPID1_PR_DCRA      (SASHW_DCR_MPIC + 0x0FC00050)
#define MPIC_IPID1_P0_DCRA      (SASHW_DCR_MPIC + 0x0FC20050)
#define MPIC_IPID1_P1_DCRA      (SASHW_DCR_MPIC + 0x0FC21050)
#define MPIC_IPID1_P2_DCRA      (SASHW_DCR_MPIC + 0x0FC22050)
#define MPIC_IPID1_P3_DCRA      (SASHW_DCR_MPIC + 0x0FC23050)

#define MPIC_IPID2_PR_DCRA      (SASHW_DCR_MPIC + 0x0FC00060)
#define MPIC_IPID2_P0_DCRA      (SASHW_DCR_MPIC + 0x0FC20060)
#define MPIC_IPID2_P1_DCRA      (SASHW_DCR_MPIC + 0x0FC21060)
#define MPIC_IPID2_P2_DCRA      (SASHW_DCR_MPIC + 0x0FC22060)
#define MPIC_IPID2_P3_DCRA      (SASHW_DCR_MPIC + 0x0FC23060)

#define MPIC_IPID3_PR_DCRA      (SASHW_DCR_MPIC + 0x0FC00070)
#define MPIC_IPID3_P0_DCRA      (SASHW_DCR_MPIC + 0x0FC20070)
#define MPIC_IPID3_P1_DCRA      (SASHW_DCR_MPIC + 0x0FC21070)
#define MPIC_IPID3_P2_DCRA      (SASHW_DCR_MPIC + 0x0FC22070)
#define MPIC_IPID3_P3_DCRA      (SASHW_DCR_MPIC + 0x0FC23070)

/***********************************************************************/
/* MPIC Current Task Priority Register (MPIC_CTP0-MPIC_CTP3)           */
/***********************************************************************/
#define MPIC_CTP_PR_DCRA        (SASHW_DCR_MPIC + 0x0FC00080)
#define MPIC_CTP_0_DCRA         (SASHW_DCR_MPIC + 0x0FC20080)
#define MPIC_CTP_1_DCRA         (SASHW_DCR_MPIC + 0x0FC21080)
#define MPIC_CTP_2_DCRA         (SASHW_DCR_MPIC + 0x0FC22080)
#define MPIC_CTP_3_DCRA         (SASHW_DCR_MPIC + 0x0FC23080)

#define MPIC_CTP_PRI_MASK       (0x0000000F)  // Processor task priority level mask
#define MPIC_CTP_PRI_SHIFT      (0)           // Processor task priority level shift

#if  !defined(__LANGUAGE_ASM__)

typedef struct _MPIC_CUR_TASK_PRIORITY
{
    union
    {
        struct
        {
            #if defined( __LITTLE_ENDIAN__ ) || defined( __LITTLE_ENDIAN )
                U32 Priority        :  4;  /*  0-3  Bits 28-31 Processor task priority level */
                U32 Reserved0to27   : 28;  /*  4-31 Bits  0-27                               */
            #else
                U32 Reserved0to27   : 28;  /*  4-31 Bits  0-27                               */
                U32 Priority        :  4;  /*  0-3  Bits 28-31 Processor task priority level */
            #endif
        }   Bits;
        U32 Word;
    } u;
} MPIC_CUR_TASK_PRIORITY, *PTR_MPIC_CUR_TASK_PRIORITY;

#endif

/***********************************************************************/
/* MPIC Who Am I Register (MPIC_WAI0-MPIC_WAI3)                        */
/***********************************************************************/
#define MPIC_WAI_PR_DCRA    (SASHW_DCR_MPIC + 0x0FC00090)
#define MPIC_WAI_0_DCRA     (SASHW_DCR_MPIC + 0x0FC20090)
#define MPIC_WAI_1_DCRA     (SASHW_DCR_MPIC + 0x0FC21090)
#define MPIC_WAI_2_DCRA     (SASHW_DCR_MPIC + 0x0FC22090)
#define MPIC_WAI_3_DCRA     (SASHW_DCR_MPIC + 0x0FC23090)

#define MPIC_WAI_ID_MASK    (0x00000003)  // Processor ID mask
#define MPIC_WAI_ID_SHIFT   (0)           // Processor ID shift

#if  !defined(__LANGUAGE_ASM__)

typedef struct _MPIC_WHO_AM_I
{
    union
    {
        struct
        {
            #if defined( __LITTLE_ENDIAN__ ) || defined( __LITTLE_ENDIAN )
                U32 ProcessorID     :  2;  /*  0-1  Bits 30-31  Processor ID */
                U32 Reserved0to29   : 30;  /*  2-31 Bits  0-29               */
            #else
                U32 Reserved0to29   : 30;  /*  2-31 Bits  0-29               */
                U32 ProcessorID     :  2;  /*  0-1  Bits 30-31  Processor ID */
            #endif
        }   Bits;
        U32 Word;
    } u;
} MPIC_MPIC_WHO_AM_I, *PTR_MPIC_WHO_AM_I;

#endif

/****************************************************************************************/
/* MPIC Interrupt Acknowledge Registers                                                 */
/*      (MPIC_NCIAR0-MPIC_NCIAR3, MPIC_CIAR0-MPIC_CIAR3, MPIC_MCIAR0-MPIC_MCIAR3)       */
/****************************************************************************************/

/* Non-critical interrupt acknowledge DCR addresses */
#define MPIC_NCIAR_PR_DCRA      (SASHW_DCR_MPIC + 0x0FC000A0)
#define MPIC_NCIAR_0_DCRA       (SASHW_DCR_MPIC + 0x0FC200A0)
#define MPIC_NCIAR_1_DCRA       (SASHW_DCR_MPIC + 0x0FC210A0)
#define MPIC_NCIAR_2_DCRA       (SASHW_DCR_MPIC + 0x0FC220A0)
#define MPIC_NCIAR_3_DCRA       (SASHW_DCR_MPIC + 0x0FC230A0)

/* Critical interrupt acknowledge DCR addresses */
#define MPIC_CIAR_PR_DCRA       (SASHW_DCR_MPIC + 0x0FC000C0)
#define MPIC_CIAR_0_DCRA        (SASHW_DCR_MPIC + 0x0FC200C0)
#define MPIC_CIAR_1_DCRA        (SASHW_DCR_MPIC + 0x0FC210C0)
#define MPIC_CIAR_2_DCRA        (SASHW_DCR_MPIC + 0x0FC220C0)
#define MPIC_CIAR_3_DCRA        (SASHW_DCR_MPIC + 0x0FC230C0)

/* Machine check interrupt acknowledge DCR addresses */
#define MPIC_MCIAR_PR_DCRA      (SASHW_DCR_MPIC + 0x0FC000E0)
#define MPIC_MCIAR_0_DCRA       (SASHW_DCR_MPIC + 0x0FC200E0)
#define MPIC_MCIAR_1_DCRA       (SASHW_DCR_MPIC + 0x0FC210E0)
#define MPIC_MCIAR_2_DCRA       (SASHW_DCR_MPIC + 0x0FC220E0)
#define MPIC_MCIAR_3_DCRA       (SASHW_DCR_MPIC + 0x0FC230E0)

#define MPIC_IAR_VEC_MASK       (0x000000FF)  // Interrupt vector mask
#define MPIC_IAR_VEC_SHIFT      (0)           // Interrupt vector shift

#if  !defined(__LANGUAGE_ASM__)

typedef struct _MPIC_INTERRUPT_ACK
{
    union
    {
        struct
        {
            #if defined( __LITTLE_ENDIAN__ ) || defined( __LITTLE_ENDIAN )
                U32 Vector          :  8;  /*  0-7  Bits 24-31  Interrupt vector */
                U32 Reserved0to23   : 24;  /*  8-31 Bits  0-23                   */
            #else
                U32 Reserved0to23   : 24;  /*  8-31 Bits  0-23                   */
                U32 Vector          :  8;  /*  0-7  Bits 24-31  Interrupt vector */
            #endif
        }   Bits;
        U32 Word;
    } u;
} MPIC_INTERRUPT_ACK, *PTR_MPIC_INTERRUPT_ACK;

#endif

/****************************************************************************************/
/* MPIC End of Interrupt Registers                                                      */
/*      (MPICx_NCEOI0-MPICx_NCEOI3, MPICx_CEOI0-MPICx_CEOI3, MPICx_MCEOI0-MPICx_MCEOI3) */
/****************************************************************************************/

/* Non-critical interrupt end of interrupt DCR addresses */
#define MPIC_NCEOI_PR_DCRA      (SASHW_DCR_MPIC + 0xFC000B0)
#define MPIC_NCEOI_0_DCRA       (SASHW_DCR_MPIC + 0xFC200B0)
#define MPIC_NCEOI_1_DCRA       (SASHW_DCR_MPIC + 0xFC210B0)
#define MPIC_NCEOI_2_DCRA       (SASHW_DCR_MPIC + 0xFC220B0)
#define MPIC_NCEOI_3_DCRA       (SASHW_DCR_MPIC + 0xFC230B0)

/* Critical interrupt end of interrupt DCR addresses */
#define MPIC_CEOI_PR_DCRA       (SASHW_DCR_MPIC + 0xFC000D0)
#define MPIC_CEOI_0_DCRA        (SASHW_DCR_MPIC + 0xFC200D0)
#define MPIC_CEOI_1_DCRA        (SASHW_DCR_MPIC + 0xFC210D0)
#define MPIC_CEOI_2_DCRA        (SASHW_DCR_MPIC + 0xFC220D0)
#define MPIC_CEOI_3_DCRA        (SASHW_DCR_MPIC + 0xFC230D0)

/* Machine check interrupt end of interrupt DCR addresses */
#define MPIC_MCEOI_PR_DCRA      (SASHW_DCR_MPIC + 0xFC000F0)
#define MPIC_MCEOI_0_DCRA       (SASHW_DCR_MPIC + 0xFC200F0)
#define MPIC_MCEOI_1_DCRA       (SASHW_DCR_MPIC + 0xFC210F0)
#define MPIC_MCEOI_2_DCRA       (SASHW_DCR_MPIC + 0xFC220F0)
#define MPIC_MCEOI_3_DCRA       (SASHW_DCR_MPIC + 0xFC230F0)

/* Writing this value to the register will signal end of interrupt processing. */
#define MPIC_END_OF_INTERRUPT   (0x0000000)

/* There is no structure or union definition for this group of registers. */

/***********************************************************************/
/* MPIC Feature Reporting Register (MPIC_FRG0)                         */
/***********************************************************************/
#define MPIC_FRG0_DCRA                  (SASHW_DCR_MPIC + 0x0FC01000)

#define MPIC_FEATURE_REPORT_VER_MASK    (0x000000FF)  // Version ID mask
#define MPIC_FEATURE_REPORT_VER_SHIFT   (0)           // Version ID shift
#define MPIC_FEATURE_REPORT_CPU_MASK    (0x00001F00)  // Number of highest CPU supported mask (3)
#define MPIC_FEATURE_REPORT_CPU_SHIFT   (8)           // Number of highest CPU supported shift
#define MPIC_FEATURE_REPORT_IRQ_MASK    (0x07FF0000)  // Number of highest IRQ source supported mask (127)
#define MPIC_FEATURE_REPORT_IRQ_SHIFT   (16)          // Number of highest IRQ source supported shift

#if  !defined(__LANGUAGE_ASM__)

typedef struct _MPIC_FEATURE_REPORT
{
    union
    {
        struct
        {
            #if defined( __LITTLE_ENDIAN__ ) || defined( __LITTLE_ENDIAN )
                U32 VersionID       :  8;  /*  0-7  Bits 24-31  Version ID                            */
                U32 MaxCPU          :  5;  /*  8-12 Bits 19-23  Number of highest CPU supported (3)   */
                U32 Reserved16to18  :  3;  /* 13-15 Bits 16-18                                        */
                U32 MaxIRQ          : 11;  /* 16-27 Bits  5-15  Number of highest IRQ supported (127) */
                U32 Reserved0to4    :  5;  /* 28-31 Bits  0- 4                                        */
            #else
                U32 Reserved0to4    :  5;  /* 28-31 Bits  0- 4                                        */
                U32 MaxIRQ          : 11;  /* 16-27 Bits  5-15  Number of highest IRQ supported (127) */
                U32 Reserved16to18  :  3;  /* 13-15 Bits 16-18                                        */
                U32 MaxCPU          :  5;  /*  8-12 Bits 19-23  Number of highest CPU supported (3)   */
                U32 VersionID       :  8;  /*  0-7  Bits 24-31  Version ID                            */
            #endif
        }   Bits;
        U32 Word;
    } u;
} MPIC_FEATURE_REPORT, *PTR_MPIC_FEATURE_REPORT;

#endif

/***********************************************************************/
/* MPIC Global Configuration Register (MPIC_GCF0)                      */
/***********************************************************************/
#define MPIC_GLOBAL_CFG_DCRA       (SASHW_DCR_MPIC + 0x0FC01020)

#define MPIC_GLOBAL_CFG_BAS_MASK   (0x0000000F)  // Base address relocation bits mask
#define MPIC_GLOBAL_CFG_BAS_SHIFT  ( 0)          // Base address relocation bits shift
#define MPIC_GLOBAL_CFG_PASS_THRU  (0x20000000)  // Pass through enable
                                                 //   0 = 8259 pass through enable
                                                 //   1 = 8259 pass through disable
#define MPIC_GLOBAL_CFG_RESET      (0x80000000)  // Reset MPIC
                                                 //   0 = No effect
                                                 //   1 = Reset controller

#if  !defined(__LANGUAGE_ASM__)

typedef struct _MPIC_GLOBAL_CFG
{
    union
    {
        struct
        {
            #if defined( __LITTLE_ENDIAN__ ) || defined( __LITTLE_ENDIAN )
                U32 BAS             :  4;  /*  0-3  Bits 28-31  Base address relocation bits       */
                U32 Reserved3to27   : 25;  /*  4-28 Bits  3-27                                     */
                U32 PassThrough     :  1;  /* 29    Bits  2     8259 interrupt pass-through enable */
                U32 Reserved1       :  1;  /* 30    Bits  1                                        */
                U32 Reset           :  1;  /* 31    Bits  0     Reset MPIC                         */
            #else
                U32 Reset           :  1;  /* 31    Bit   0     Reset MPIC                         */
                U32 Reserved1       :  1;  /* 30    Bit   1                                        */
                U32 PassThrough     :  1;  /* 29    Bit   2     8259 interrupt pass-through enable */
                U32 Reserved3to27   : 25;  /*  4-28 Bits  3-27                                     */
                U32 BAS             :  4;  /*  0-3  Bits 28-31  Base address relocation bits       */
            #endif
        }   Bits;
        U32 Word;
    } u;
} MPIC_GLOBAL_CFG, *PTR_MPIC_GLOBAL_CFG;

#endif

/***********************************************************************/
/* MPIC Vendor Interrupt Type Configuration Register (MPIC_VITC)       */
/***********************************************************************/
#define MPIC_VITC_DCRA       (SASHW_DCR_MPIC + 0x0FC01040)

#define MPIC_VITC_CRB_MASK   (0x0000001F)
#define MPIC_VITC_CRB_SHIFT  ( 0)
#define MPIC_VITC_MCB_MASK   (0x00001F00)
#define MPIC_VITC_MCB_SHIFT  ( 8)

#if  !defined(__LANGUAGE_ASM__)

typedef struct _MPIC_IRQ_TYPE_CFG
{
    union
    {
        struct
        {
            #if defined( __LITTLE_ENDIAN__ ) || defined( __LITTLE_ENDIAN )
                U32 LowCritIrqBorder    :   5;  /*  0-4  Bits 27-31  Lower critical interrupt border       */
                U32 Reserved24to26      :   3;  /*  5-7  Bits 24-26                                        */
                U32 LowerMachChkBorder  :   5;  /*  8-12 Bits 19-23  Lower machine check interrupt border  */
                U32 Reserved0to18       :  19;  /* 13-31 Bits  0-18                                        */
            #else
                U32 Reserved0to18       :  19;  /* 13-31 Bits  0-18                                        */
                U32 LowerMachChkBorder  :   5;  /*  8-12 Bits 19-23  Lower machine check interrupt border  */
                U32 Reserved24to26      :   3;  /*  5-7 Bits 24-26                                        */
                U32 LowCritIrqBorder    :   5;  /*  0-4 Bits 27-31  Lower critical interrupt border       */
            #endif
        }   Bits;
        U32 Word;
    } u;
} MPIC_IRQ_TYPE_CFG, *PTR_MPIC_IRQ_TYPE_CFG;

#endif

/***********************************************************************/
/* MPIC Vendor Raw Interrupt Debug Register (MPICx_VRID)               */
/***********************************************************************/
#define MPIC_VRID_DCRA                  (SASHW_DCR_MPIC + 0x0FC01050)  // DCR address

#define MPIC_RAW_IRQ_DBG_GROUP_MASK     (0x00000007)  // Interrupt group mask
#define MPIC_RAW_IRQ_DBG_GROUP_SHIFT    ( 0)          // Interrupt group shift
#define MPIC_RAW_IRQ_DBG_IPB_MASK       (0xFFFFF000)  // Interrupt pending register section mask
#define MPIC_RAW_IRQ_DBG_IPB_SHIFT      (12)          // Interrupt pending register section shift

/* Values for the group field */
#define MPIC_RAW_IRQ_DBG_GROUP_0_19     (0)           // IRQs 0-19
#define MPIC_RAW_IRQ_DBG_GROUP_20_39    (1)           // IRQs 20-39
#define MPIC_RAW_IRQ_DBG_GROUP_40_59    (2)           // IRQs 40-59
#define MPIC_RAW_IRQ_DBG_GROUP_60_79    (3)           // IRQs 60-79
#define MPIC_RAW_IRQ_DBG_GROUP_80_99    (4)           // IRQs 80-99
#define MPIC_RAW_IRQ_DBG_GROUP_100_119  (5)           // IRQs 100-119
#define MPIC_RAW_IRQ_DBG_GROUP_120_127P (6)           // IRQs 120-127, TMR0-3, IPI0-3

#if  !defined(__LANGUAGE_ASM__)

typedef struct _MPIC_RAW_IRQ_DEBUG
{
    union
    {
        struct
        {
            #if defined( __LITTLE_ENDIAN__ ) || defined( __LITTLE_ENDIAN )
                U32 Group           :   3;  /*  0-2  Bits 29-31  Interrupt group                    */
                U32 Reserved20to28  :   9;  /*  4-11 Bits 20-28                                     */
                U32 IrqPending      :  20;  /* 12-31 Bits  0-19  Interrupt pending register section */
            #else
                U32 IrqPending      :  20;  /* 12-31 Bits  0-19  Interrupt group                    */
                U32 Reserved20to28  :   9;  /*  4-11 Bits 20-28                                     */
                U32 Group           :   3;  /*  0-2  Bits 29-31  Interrupt pending register section */
            #endif
        }   Bits;
        U32 Word;
    } u;
} MPIC_RAW_IRQ_DEBUG, *PTR_MPIC_RAW_IRQ_DEBUG;

#endif

/***********************************************************************/
/* MPIC Vendor Soft Core Revision ID Register (MPICx_REVID)            */
/***********************************************************************/
#define MPIC_REVID_DCRA       (SASHW_DCR_MPIC + 0x0FC01070)  // DCR address

#define MPIC_REVID_BRN_MASK   (0x000000FF)  // Branch number mask
#define MPIC_REVID_BRN_SHIFT  ( 0)          // Branch number shift
#define MPIC_REVID_RN_MASK    (0x000FFF00)  // Revision number mask
#define MPIC_REVID_RN_SHIFT   ( 8)          // Revision number shift

#if  !defined(__LANGUAGE_ASM__)

typedef struct _MPIC_REVID
{
  union
  {
    struct
    {
        #if defined( __LITTLE_ENDIAN__ ) || defined( __LITTLE_ENDIAN )
            U32 BranchNum       :   8;  /*  0-7  Bits 24-31  Branch number   */
            U32 RevNum          :  12;  /*  8-19 Bits 12-23  Revision number */
            U32 Reserved0to11   :  12;  /* 20-31 Bits  0-11                  */
        #else
            U32 Reserved0to11   :  12;  /* 20-31 Bits  0-11                  */
            U32 RevNum          :  12;  /*  8-19 Bits 12-23  Revision number */
            U32 BranchNum       :   8;  /*  0-7  Bits 24-31  Branch number   */
        #endif
    }   Bits;
    U32 Word;
  } u;
} MPIC_REVID, *PTR_MPIC_REVID;

#endif

/***********************************************************************/
/* MPIC Vendor Identification Register (MPICx_VIR)                     */
/***********************************************************************/
#define MPIC_VEND_ID_DCRA       (SASHW_DCR_MPIC + 0x0FC01080)  // DCR address

#define MPIC_VEND_ID_VID_MASK   (0x000000FF)  // Vendor ID mask (IBM = 0x14)
#define MPIC_VEND_ID_VID_SHIFT  ( 0)          // Vendor ID shift
#define MPIC_VEND_ID_DEV_MASK   (0x0000FF00)  // Device ID Mask (IBM = 0x50)
#define MPIC_VEND_ID_DEV_SHIFT  ( 8)          // Device ID shift
#define MPIC_VEND_ID_STP_MASK   (0x00FF0000)  // Stepping mask
#define MPIC_VEND_ID_STP_SHIFT  (16)          // Stepping shift

#if  !defined(__LANGUAGE_ASM__)

typedef struct _MPIC_VEND_ID
{
    union
    {
        struct
        {
            #if defined( __LITTLE_ENDIAN__ ) || defined( __LITTLE_ENDIAN )
                U32 VendorID        :   8;  /*  0-7  Bits 24-31  Vendor ID */
                U32 DeviceID        :   8;  /*  8-15 Bits 16-23  Device ID */
                U32 Stepping        :   8;  /* 16-23 Bits  8-15  Stepping  */
                U32 Reserved0to7    :   8;  /* 24-31 Bits  0-7             */
            #else
                U32 Reserved0to7    :   8;  /* 24-31 Bits  0-7             */
                U32 Stepping        :   8;  /* 16-23 Bits  8-15  Stepping  */
                U32 DeviceID        :   8;  /*  8-15 Bits 16-23  Device ID */
                U32 VendorID        :   8;  /*  0-7  Bits 24-31  Vendor ID */
            #endif
        }   Bits;
        U32 Word;
    } u;
} MPIC_VEND_ID, * PTR_MPIC_VEND_ID;

#endif

/***********************************************************************/
/* MPIC Processor Initialization  Register (MPICx_PINI)                */
/***********************************************************************/

/*
 * NOTE: This register uses the Processor Select bit and structure
 *       ( _MPIC_PROCESSOR_SELECT ) definitions below.
 */
#define MPIC_PROC_INIT_DCRA  (SASHW_DCR_MPIC + 0x0FC01090)  // DCR address

/***********************************************************************/
/* MPIC IPI Vector/Priority Registers (MPICx_MIVP0-MPICx_MIVP3)        */
/***********************************************************************/
#define MPIC_IPI_VECTOR_PRI_0_DCRA     (SASHW_DCR_MPIC + 0x0FC010A0)  // MPICx_IPV0 DCR address
#define MPIC_IPI_VECTOR_PRI_1_DCRA     (SASHW_DCR_MPIC + 0x0FC010B0)  // MPICx_IPV1 DCR address
#define MPIC_IPI_VECTOR_PRI_2_DCRA     (SASHW_DCR_MPIC + 0x0FC010C0)  // MPICx_IPV2 DCR address
#define MPIC_IPI_VECTOR_PRI_3_DCRA     (SASHW_DCR_MPIC + 0x0FC010D0)  // MPICx_IPV3 DCR address

#define MPIC_IPI_VECTOR_PRI_VEC_MASK   (0x000000FF)  // Interrupt vector mask
#define MPIC_IPI_VECTOR_PRI_VEC_SHIFT  ( 0)          // Interrupt vector shift
#define MPIC_IPI_VECTOR_PRI_PRI_MASK   (0x000F0000)  // Interrupt priority mask
#define MPIC_IPI_VECTOR_PRI_PRI_SHIFT  (16)          // Interrupt priority shift
#define MPIC_IPI_VECTOR_PRI_ACT        (0x40000000)  // Activity
                                                     //   0 = Okay to modify this register
                                                     //   1 = In use; do not modify
#define MPIC_IPI_VECTOR_PRI_MASK        (0x80000000)  // Mask
                                                      //   0 = Interrupt enabled
                                                      //   1 = Further interrupts disabled

#if  !defined(__LANGUAGE_ASM__)

typedef struct _MPIC_IPI_VECTOR_PRI
{
    union
    {
        struct
        {
            #if defined( __LITTLE_ENDIAN__ ) || defined( __LITTLE_ENDIAN )
                U32 Vector          :   8;  /*  0-7  Bits 24-31  Interrupt vector   */
                U32 Reserved16to23  :   8;  /*  8-15 Bits 16-23                     */
                U32 Priority        :   4;  /* 16-19 Bits 12-15  Interrupt priority */
                U32 Reserved2to11   :  10;  /* 20-29 Bits  2-11                     */
                U32 Activity        :   1;  /* 30    Bits  1     Activity           */
                U32 Mask            :   1;  /* 31    Bits  0     Mask               */
            #else
                U32 Mask            :   1;  /* 31    Bits  0     Mask               */
                U32 Activity        :   1;  /* 30    Bits  1     Activity           */
                U32 Reserved2to11   :  10;  /* 20-29 Bits  2-11                     */
                U32 Priority        :   4;  /* 16-19 Bits 12-15  Interrupt priority */
                U32 Reserved16to23  :   8;  /*  8-15 Bits 16-23                     */
                U32 Vector          :   8;  /*  0-7  Bits 24-31  Interrupt vector   */
            #endif
        }   Bits;
        U32 Word;
    } u;
} MPIC_IPI_VECTOR_PRI, *PTR_MPIC_IPI_VECTOR_PRI;

#endif

/***********************************************************************/
/* MPIC Spurious Vector Register (MPICx_SPV)                           */
/***********************************************************************/
#define MPIC_SPURIOUS_VECTOR_DCRA       (SASHW_DCR_MPIC + 0x0FC010E0)  // DCR address

#define MPIC_SPURIOUS_VECTOR_VEC_MASK   (0x000000FF)  // Interrupt vector mask
#define MPIC_SPURIOUS_VECTOR_VEC_SHIFT  ( 0)          // Interrupt vector shift

#if  !defined(__LANGUAGE_ASM__)

typedef struct _MPIC_SPURIOUS_VECTOR
{
    union
    {
        struct
        {
            #if defined( __LITTLE_ENDIAN__ ) || defined( __LITTLE_ENDIAN )
                U32 Vector          :   8;  /*  0-7  Bits 24-31  Spurious Interrupt vector */
                U32 Reserved0to23   :  24;  /*  8-31 Bits  0-23                            */
            #else
                U32 Reserved0to23   :  24;  /*  8-31 Bits  0-23                            */
                U32 Vector          :   8;  /*  0-7  Bits 24-31  Spurious Interrupt vector */
            #endif
        }   Bits;
        U32 Word;
    } u;
} MPIC_SPURIOUS_VECTOR, *PTR_MPIC_SPURIOUS_VECTOR;

#endif

/***********************************************************************/
/* MPIC Timer Frequency Reporting Register (MPICx_TFR)                 */
/***********************************************************************/
#define MPIC_TMR_FRQ_RPT_DCRA      (SASHW_DCR_MPIC + 0x0FC010F0)

/* There is no  definitions for this register */

/***********************************************************************/
/* MPIC Timer n Current Count Registers (MPICx_TCC0-MPICx_TCC3)        */
/***********************************************************************/
#define MPIC_TMR_COUNT_0_DCRA       (SASHW_DCR_MPIC + 0x0FC01100)  // MPICx_TCC0 DCR address
#define MPIC_TMR_COUNT_1_DCRA       (SASHW_DCR_MPIC + 0x0FC01140)  // MPICx_TCC1 DCR address
#define MPIC_TMR_COUNT_2_DCRA       (SASHW_DCR_MPIC + 0x0FC01180)  // MPICx_TCC2 DCR address
#define MPIC_TMR_COUNT_3_DCRA       (SASHW_DCR_MPIC + 0x0FC011C0)  // MPICx_TCC3 DCR address

#define MPIC_TMR_COUNT_CC_MASK      (0x7FFFFFFF)  // Timer current count mask
#define MPIC_TMR_COUNT_CC_SHIFT     (0)
#define MPIC_TMR_COUNT_T            (0x800000000) // Toggle

#if  !defined(__LANGUAGE_ASM__)

typedef struct _MPIC_TMR_COUNT
{
    union
    {
        struct
        {
            #if defined( __LITTLE_ENDIAN__ ) || defined( __LITTLE_ENDIAN )
                U32 Count           :  31;  /*  0-30 Bits 1-31  Timer current count */
                U32 Toggle          :   1;  /* 31    Bits 0     Toggle              */
            #else
                U32 Toggle          :   1;  /* 31    Bits 0     Toggle              */
                U32 Count           :  31;  /*  0-30 Bits 1-31  Timer current count */
            #endif
        }   Bits;
        U32 Word;
    } u;
} MPIC_TMR_COUNT, *PTR_MPIC_TMR_COUNT;

#endif

/***********************************************************************/
/* MPIC Timer n Base Count Registers (MPICx_TBC0-MPICx_TBC3)           */
/***********************************************************************/
#define MPIC_TMR_BASE_CNT_0_DCRA    (SASHW_DCR_MPIC + 0x0FC01110)
#define MPIC_TMR_BASE_CNT_1_DCRA    (SASHW_DCR_MPIC + 0x0FC01150)
#define MPIC_TMR_BASE_CNT_2_DCRA    (SASHW_DCR_MPIC + 0x0FC01190)
#define MPIC_TMR_BASE_CNT_3_DCRA    (SASHW_DCR_MPIC + 0x0FC011D0)

#define MPIC_TMR_BASE_CNT_BC_MASK   (0x7FFFFFFF)
#define MPIC_TMR_BASE_CNT_BC_SHIFT  (0)
#define MPIC_TMR_BASE_CNT_CI        (0x80000000)

#if  !defined(__LANGUAGE_ASM__)

typedef struct _MPIC_TMR_BASE_CNT
{
    union
    {
        struct
        {
            #if defined( __LITTLE_ENDIAN__ ) || defined( __LITTLE_ENDIAN )
                U32 BaseCount       :   31;  /*  0-30 Bits 1-31  Timer base count */
                U32 CountInhibit    :    1;  /* 31    Bits 0     Count inhibit    */
            #else
                U32 CountInhibit    :    1;  /* 31    Bits 0     Count inhibit    */
                U32 BaseCount       :   31;  /*  0-30 Bits 1-31  Timer base count */
            #endif
        }   Bits;
        U32 Word;
    } u;
} MPIC_TMR_BASE_CNT, * PTR_MPIC_TMR_BASE_CNT;

#endif

/***********************************************************************/
/* MPIC Timer n Vector/Priority Registers (MPICx_TVP0-MPICx_TVP3)      */
/***********************************************************************/
#define MPIC_TMR_VEC_PRI_0_DCRA     (SASHW_DCR_MPIC + 0x0FC01120)  // MPICx_TVP0 DCR address
#define MPIC_TMR_VEC_PRI_1_DCRA     (SASHW_DCR_MPIC + 0x0FC01160)  // MPICx_TVP1 DCR address
#define MPIC_TMR_VEC_PRI_2_DCRA     (SASHW_DCR_MPIC + 0x0FC011A0)  // MPICx_TVP2 DCR address
#define MPIC_TMR_VEC_PRI_3_DCRA     (SASHW_DCR_MPIC + 0x0FC011E0)  // MPICx_TVP3 DCR address
#define MPIC_TMR_VEC_PRI_INCR       (0x40)                         // MPICx_TVP Incriment

#define MPIC_VECTOR_PRIORITY_TIMER_DCRA(n)   (MPIC_TMR_VEC_PRI_0_DCRA + ((n) * MPIC_TMR_VEC_PRI_INCR))

#define MPIC_TMR_VEC_PRI_VEC_MASK   (0x000000FF)  // Interrupt vector mask
#define MPIC_TMR_VEC_PRI_VEC_SHIFT  ( 0)          // Interrupt vector shift
#define MPIC_TMR_VEC_PRI_PRI_MASK   (0x000F0000)  // Interrupt priority mask
#define MPIC_TMR_VEC_PRI_PRI_SHIFT  (12)          // Interrupt priority shift
#define MPIC_TMR_VEC_PRI_ACT        (0x40000000)  // Activity
                                                  //   0 = OKay to modify this register
                                                  //   1 = In use; do not modify
#define MPIC_TM_VEC_PRI_MASK        (0x80000000)  // Mask
                                                  //   0 = Interrupt enabled
                                                  //   1 = Further interrupts disabled

#if  !defined(__LANGUAGE_ASM__)

typedef struct _MPIC_TMR_VEC_PRI
{
    union
    {
        struct
        {
            #if defined( __LITTLE_ENDIAN__ ) || defined( __LITTLE_ENDIAN )
                U32 Vector          :   8;  /*  0-7  Bits 24-31  Interrupt vector    */
                U32 Reserved16to23  :   8;  /*  8-15 Bits 16-23                      */
                U32 Priority        :   4;  /* 16-19 Bits 12-15  Interrupt priority  */
                U32 Reserved2to11   :  10;  /* 20-29 Bits  2-11                      */
                U32 Activity        :   1;  /* 30    Bits  1     Activity            */
                U32 Mask            :   1;  /* 31    Bits  0     Mask                */
            #else
                U32 Mask            :   1;  /* 31    Bits  0     Mask                */
                U32 Activity        :   1;  /* 30    Bits  1     Activity            */
                U32 Reserved2to11   :  10;  /* 20-29 Bits  2-11                      */
                U32 Priority        :   4;  /* 16-19 Bits 12-15  Interrupt priority  */
                U32 Reserved16to23  :   8;  /*  8-15 Bits 16-23                      */
                U32 Vector          :   8;  /*  0-7  Bits 24-31  Interrupt vector    */
            #endif
        }   Bits;
        U32 Word;
    } u;
} MPIC_TMR_VEC_PRI, *PTR_MPIC_TMR_VEC_PRI;

#endif

/***********************************************************************/
/* MPIC Timer n Destination Registers (MPICx_TDST0-MPICx_TDST3)        */
/***********************************************************************/
/*
 * NOTE: These registers use the Processor Select bit and structure
 *       ( _MPIC_PROCESSOR_SELECT ) definitions below.
 */
#define MPIC_TMR_DEST_0_DCRA  (SASHW_DCR_MPIC + 0x0FC01130)  // MPICx_TDST0 DCR address
#define MPIC_TMR_DEST_1_DCRA  (SASHW_DCR_MPIC + 0x0FC01170)  // MPICx_TDST1 DCR address
#define MPIC_TMR_DEST_2_DCRA  (SASHW_DCR_MPIC + 0x0FC011B0)  // MPICx_TDST2 DCR address
#define MPIC_TMR_DEST_3_DCRA  (SASHW_DCR_MPIC + 0x0FC011F0)  // MPICx_TDST3 DCR address
#define MPIC_TMR_DEST_INCR    (0x40)

#define MPIC_DST_TIMER_DCRA(n)  ( MPIC_TMR_DEST_0_DCRA + ((n) * MPIC_TMR_DEST_INCR) )

/********************************************************
* MPIC Processor Select Masks and Structure Definitions *
********************************************************/

/* These bit and structure definitions are used for the following registers:
 *     - MPICx_DST0-MPICx_DST127
 *     - MPICx_IPID0-MPICx_IPID3
 *     - MPICx_PINI
 *     - MPICx_TDST0-MPICx_TDST3
 */
#define MPIC_PROC0_SELECT   (0x00000001)  // Processor 0 select
#define MPIC_PROC1_SELECT   (0x00000002)  // Processor 1 select
#define MPIC_PROC2_SELECT   (0x00000004)  // Processor 2 select
#define MPIC_PROC3_SELECT   (0x00000008)  // Processor 3 select

#if  !defined(__LANGUAGE_ASM__)

typedef struct _MPIC_PROCESSOR_SELECT
{
    union
    {
        struct
        {
            #if defined( __LITTLE_ENDIAN__ ) || defined( __LITTLE_ENDIAN )
                U32 Proc0Selected   :  1;  /*  0    bits 31    Processor 0 select  */
                U32 Proc1Selected   :  1;  /*  1    bits 30    Processor 1 select  */
                U32 Proc2Selected   :  1;  /*  2    bits 29    Processor 2 select  */
                U32 Proc3Selected   :  1;  /*  3    bits 28    Processor 3 select  */
                U32 Reserved0to27   : 28;  /*  4-31 bits  0-27                     */
            #else
                U32 Reserved0to27   : 28;  /*  4-31 bits  0-27                     */
                U32 Proc3Selected   :  1;  /*  3    bits 28    Processor 3 select  */
                U32 Proc2Selected   :  1;  /*  2    bits 29    Processor 2 select  */
                U32 Proc1Selected   :  1;  /*  1    bits 30    Processor 1 select  */
                U32 Proc0Selected   :  1;  /*  0    bits 31    Processor 0 select  */
            #endif
        }   Bits;
        U32 Word;
    } u;
} MPIC_PROCESSOR_SELECT, *PTR_MPIC_PROCESSOR_SELECT;

#endif

#endif /* __MPIC_H_INCLUDED */

