/***************************************************************************
 *                                                                         *
 *  Copyright (C) 2012 LSI Corporation.  All rights reserved.              *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation. The   *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  osalKernel.c
 *          Title:  Wrappers to the kernel functions
 *     Programmer:  Naresh Bhalala
 *  Creation Date:  April 27, 2012
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   --------------------------------------------------------
 *  04/27/12  NB    Initial version.
 *  04/30/12  NB    Made common for peripherals - Flash, I2C, Ethernet, GPIO, UART
 *
 *
 *  Description
 *  ------------
 *  This file contains the implementation of the wrapper functions to
 *  the OS functions.
 *
 *
 * -------------------------------------------------------------------------
 */

#ifndef _OSAL_KERNEL_C_SOURCE
#define _OSAL_KERNEL_C_SOURCE

#include <asm/io.h>
#include <asm/dcr-native.h>
#include <asm/reg.h>
#include <asm/pgtable.h>
#include <linux/kernel.h>
#include <linux/delay.h>
#include <stdarg.h>

#include "osalKernel.h"

/* Macro to write DCR space */
#define MoveToDCR(Value, DCRAddress)      \
    __asm__ volatile ("mtdcrx  %0,%1" : : "r"(DCRAddress), "r"(Value));

/* Macro to read DCR space */
#define MoveFromDCR(DCRAddress)      \
    __asm__ volatile ("mfdcrx  %0,%1" : "=r"(ret) : "r"(DCRAddress));

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: osalPrintk()
* SUMMARY: Print the data on console.
* SCOPE: global.
*
*
* DESCRIPTION:
* Print the data on console.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* Format - constant formatted string value.
*
* RETURNS:
* void
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES:
*
----------------------------------------------------------------------------*/
void osalPrintk ( const char* format, ... )
{
    va_list args;

    va_start(args, format);
    vprintk(format, args);
    va_end(args);
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: osalIoremap()
* SUMMARY: Map the IO to the kernel space.
* SCOPE: global.
*
*
* DESCRIPTION:
* Map the IO to the kernel space.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* Address   - Pointer to the IO region
* SizeBytes - Length of region to be mapped
*
* RETURNS:
* void *    Pointer at which the IO is mapped
* NULL      null pointer
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES:
*
----------------------------------------------------------------------------*/
void * osalIoremap ( unsigned long long Address, U32 SizeBytes )
{
    void * addr;

    addr = (void *)ioremap(Address, SizeBytes);
    if (!addr)
    {
        return NULL;
    }

    return addr;
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: osalIounmap()
* SUMMARY: Un-map the IO from the kernel space.
* SCOPE: global.
*
*
* DESCRIPTION:
* Un-map the IO from the kernel space.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* Address - Pointer to the memory where IO is mapped
*
* RETURNS:
* void
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES:
*
----------------------------------------------------------------------------*/
void osalIounmap ( void * Address )
{
    iounmap(Address);
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: osalUdelay()
* SUMMARY: Generate delays in micro-seconds
* SCOPE: global.
*
*
* DESCRIPTION:
* Generate delays in micro-seconds
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* Delay - delay value in micro-second
*
* RETURNS:
* void
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES:
*
----------------------------------------------------------------------------*/
void osalUdelay ( U32 Delay )
{
    udelay(Delay);
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: osalMdelay()
* SUMMARY: Generate delays in milliseconds
* SCOPE: global.
*
* DESCRIPTION:
* Generate delays in milliseconds
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* Delay - delay value in millisecond
*
* RETURNS:
* void
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES:
*
----------------------------------------------------------------------------*/
void osalMdelay ( Ul32 Delay )
{
    mdelay(Delay); /* Delay value in milliseconds */
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: osalConvertPhysicalToVirtual()
* SUMMARY: Converts physical address to virtual.
* SCOPE: global
*
*
* DESCRIPTION:
* Converts physical address to virtual.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* Address - Physical address to be converted to virtual.
*
* RETURNS:
* void *    Pointer to the virtual memory space.
* NULL      null pointer.
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES:
*
----------------------------------------------------------------------------*/
void * osalConvertPhysicalToVirtual ( unsigned long long Address )
{
    void * addr;

    addr = phys_to_virt(Address);
    if (!addr)
    {
        return NULL;
    }

    return addr;
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: osalMoveFromDcr()
* SUMMARY: Moves the contents of DCR.
* SCOPE: global
*
*
* DESCRIPTION:
* Moves the contents of DCR.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* Address - Address of the register whose contents are to be read.
*
* RETURNS:
* U32 value i.e. the contents read from the DCR.
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES:
*
----------------------------------------------------------------------------*/
U32 osalMoveFromDcr ( S32 RegAddress )
{
    U32 ret;

    MoveFromDCR(RegAddress);
    return ret;
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: osalMoveToDcr()
* SUMMARY: Moves the contents to DCR.
* SCOPE: global
*
*
* DESCRIPTION:
* Moves the contents to DCR.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* RegAddress - Address of the register where the value has to be written.
* Value    - Value to be written.
*
* RETURNS:
* void
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES:
*
----------------------------------------------------------------------------*/
void osalMoveToDcr ( U32 RegAddress, U32 Value )
{
    MoveToDCR(RegAddress, Value);
    return;
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: osalMemset()
* SUMMARY: Initialize the memory space with given value.
* SCOPE: global.
*
*
* DESCRIPTION:
* Initialize the memory space with given value.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* Address - Pointer to the memory to be initialized.
* Value   - value with the memory to be initialized.
* Count   - Length of memory to be memset'ed.
*
* RETURNS:
* void
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES:
*
----------------------------------------------------------------------------*/
void osalMemset ( void * Address, U8 Value, U32 Count )
{
    memset(Address, Value, Count);
}

#endif /* _OSAL_KERNEL_C_SOURCE */