/***************************************************************************
 *                                                                         *
 *  Copyright (C) 2012 LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  raidAccelerator.h
 *          Title:  Library Header File
 *     Programmer:  Amit M Jain
 *  Creation Date:  06/08/2010
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   --------------------------------------------------------
 *  06/08/10  AMJ   Initial version.
 *  06/23/10  JM    Code restructured as per new design
 *  07/21/11  NKB   This library is linked to the RAID ENGINE Driver.
 *  01/28/12  JM    RAID Accelerator header file is modified for LSISAS3108.
 *  04/17/12  NB    RAID code restructured for LSISAS3108.
 *
 * Description
 * ------------
 *  This file contains the definitions and the function prototypes of the
 *  functions implemented in the raid accelerator library.
 *
 *--------------------------------------------------------------------------
 */

#ifndef BLDLIB_H_SOURCE
#define BLDLIB_H_SOURCE

#define MAX_RACC_DMA                        (0x02)
#define RA_DMA_0                            (0x00)
#define RA_DMA_1                            (0x01)

#define LINK_LIST_A                         (0x00)
#define LINK_LIST_B                         (0x01)
#define LINK_LIST_AB                        (0x02)

#define MAX_NUMBER_OF_CDB                   (2048)

/* DMA Operations */
#define DMA_OP_MASK                         (0x000000ff)
#define DMA_OP_NOP                          (0x00)
#define DMA_OP_MOVE_SINGLE                  (0x01)
#define DMA_OP_MULTICAST_MOVE               (0x02)
#define DMA_OP_MULTISOURCE_MOVE             (0x03)
/* opcodes 4 and 5 are reserved */
#define DMA_OP_GENERATE_XOR1                (0x06)
#define DMA_OP_GENERATE_XOR1_XOR2           (0x08)
#define DMA_OP_PATTERN_CHECK                (0x09)
#define DMA_OP_PATTERN_FILL                 (0x0A)
#define DMA_OP_LFSR_CHECK                   (0x0B)
#define DMA_OP_LFSR_FILL                    (0x0C)
#define DMA_OP_GENERATE_HASH                (0x0D)
#define DMA_OP_GENERATE_HMAC                (0x0E)
#define DMA_OP_MOVE_WITH_MULTIPLY           (0x0F)

typedef struct _RACC_DMA_SOURCE_EEDP_PAIR
{
    RACC_DMA_SOURCE        Src;
    RACC_DMA_SOURCE_EEDP   Eedp;
} RACC_DMA_SOURCE_EEDP_PAIR, * PTR_RACC_DMA_SOURCE_EEDP_PAIR;

/* Following CDB contain fields for Data Protection attributes */
/* XOR1 CDB with EEDP attributes */
typedef struct _XOR1_EEDP_CDB
{
    RACC_DMA_CDB_HEADER         Header;         /* 0-0x1F    */
    RACC_DMA_SOURCE_EEDP_PAIR   SrcEedp[2];     /* 0x20-0x5F */
    RACC_DMA_XOR1               Xor1;           /* 0x60-0x6F */
} XOR1_EEDP_CDB, * PTR_XOR1_EEDP_CDB;

/* XOR2 CDB with EEDP attributes */
typedef struct _XOR2_EEDP_CDB
{
    RACC_DMA_CDB_HEADER         Header;         /* 0-0x1F    */
    RACC_DMA_SOURCE_EEDP_PAIR   SrcEedp[2];     /* 0x20-0x5F */
    RACC_DMA_XOR1               Xor1;           /* 0x60-0x6F */
    RACC_DMA_XOR2               Xor2;           /* 0x70-0x7F */
} XOR2_EEDP_CDB, * PTR_XOR2_EEDP_CDB;

/* Generate Hash CDB with EEDP attributes */
typedef struct _GEN_HASH_EEDP_CDB
{
    RACC_DMA_CDB_HEADER          Header;                 /* 0-0x1F    */
    RACC_DMA_SOURCE_EEDP_PAIR    SrcEedp;                /* 0x20-0x3F */
    U64                          DestinationAddress;     /* 0x40-0x47 */
    U8                           DestinationAttributes;  /* 0x48      */
    U8                           DestinationReserved[7]; /* 0x49-0x4F */
} GEN_HASH_EEDP_CDB, * PTR_GEN_HASH_EEDP_CDB;

/* Generate HMAC CDB with EEDP attributes */
typedef struct _GEN_HMAC_EEDP_CDB
{
    RACC_DMA_CDB_HEADER          Header;                 /* 0-0x1F    */
    RACC_DMA_SOURCE_EEDP_PAIR    SrcEedp;                /* 0x20-0x3F */
    U64                          HashKeyPointer;         /* 0x40-0x47 */
    U64                          Reserved1;              /* 0x48-0x4F */
    U64                          DestinationAddress;     /* 0x50-0x57 */
    U8                           DestinationAttributes;  /* 0x58      */
    U8                           DestinationReserved[7]; /* 0x59-0x5f */
} GEN_HMAC_EEDP_CDB, * PTR_GEN_HMAC_EEDP_CDB;

/* RAID CDB union for different types of CDB */
typedef union _RAID_CDB_UNION
{
    NOOP_CDB          NoOp;
    SSM_CDB_ALIGNED   SingleSrcMove;
    SSM_EEDP_CDB      SingleSrcMoveEedp;
    MCM_CDB           MultiCastMove;
    XOR1_CDB          Xor1;
    XOR1_EEDP_CDB     Xor1Eedp;
    XOR2_CDB          Xor2;
    XOR2_EEDP_CDB     Xor2Eedp;
    CHK_PAT_CDB       CheckPattern;
    FILL_PAT_CDB      FillPattern;
    CHK_LFSR_CDB      CheckLFSR;
    FILL_LFSR_CDB     FillLFSR;
    GEN_HASH_CDB      GenerateHash;
    GEN_HASH_EEDP_CDB GenerateHashEedp;
    GEN_HMAC_CDB      GenerateHMAC;
    GEN_HMAC_EEDP_CDB GenerateHMACEedp;
    MM_CDB            MoveMultiply;
} RAID_CDB_UNION, * PTR_RAID_CDB_UNION;

/* Return status of APIs */
typedef enum _RA_STATUS
{
    RA_SUCCESS = 0,
    RA_FAILURE,
    RA_INVALID_ARGUMENT,
    RA_INIT_FAILED,
    RA_DMA_NOT_INITIALIZED,
    RA_DMA_ARBITRATION_FAILED,
    RA_DMA_SETUP_TRANSFER_FAILED,
    RA_DMA_COMMAND_PROCESSING_FAILED
} RA_STATUS;

/* Following flag is used by start command processing API */
typedef enum _RA_CDB_START_XFER_FLAG
{
    NEW_CDB_ROOT_POINTER_FLG = 0,
    NEW_CDB_POINTER_FLG
} RA_CDB_START_XFER_FLAG;

/* This structure needs to filled up by Application/Driver before using
 * any APIs
 */
typedef struct _RAID_ACCELERATOR_BUFFER
{
    /* Total number of CDB from all link list which will process by
     * both DMA
     */
    U8  TotalNumberOfCdb;

    /* Size of the Completion queue which will use during RAID
     * initialization API
     */
    U16 CompletionQSize[MAX_RACC_DMA];

    /* Base address of Completion queue which will use during RAID
     * initialization API
     */
    U32 CompletionQBaseAddress[MAX_RACC_DMA];

    /* Valid entry check for processed CDB during the reading of the
     * Completion queue. It needs to set TRUE before used by API.
     */
    U8  ValidCompletionEntry[MAX_RACC_DMA];

    /* After setting up the Link List of CDB with DMA, set the DmaEnable[] as
     * TRUE for CDB processing from the specified DMA number.
     */
    U32 DmaEnable[MAX_RACC_DMA];

    /* After setting up the Link List of CDB with DMA, set the link list type
     * as LINK_LIST_A or LINK_LIST_B or LINK_LIST_AB for the CDB processing
     * from the specified link list.
     */
    U32 LinkListType[MAX_RACC_DMA];
} RAID_ACCELERATOR_BUFFER, * PTR_RAID_ACCELERATOR_BUFFER;

/* DMA errors logged in this structure by API */
typedef struct _RA_DMA_ERROR_REGS
{
    U32                           ErrorCDBLowerAddrPointer;
    RACC_DMA_HALT_ERR_ATTR_REG    ErrorAttributes;
    RACC_DMA_HALT_ERR_STAT_REG    ErrorStatus;
    U32                           FailedOpCode;
} RA_DMA_ERROR_REGS, * PTR_RA_DMA_ERROR_REGS;

/* Processed CDB status logged in this structure by API */
typedef struct _RA_DMA_CDB_STATUS
{
    U32 Status;
    U32 Opcode;
    U32 LinkedList;
    U32 DmaNum;
} RA_DMA_CDB_STATUS, * PTR_RA_DMA_CDB_STATUS;

/* This structure is used by start command processing API */
typedef struct _RA_DMA_COMMAND_PROCESS
{
    U32 TimeOut;
    PTR_RA_DMA_ERROR_REGS PtrdDma0ConfigRegs;
    PTR_RA_DMA_ERROR_REGS PtrdDma1ConfigRegs;
    U32 Dma0ErrorOccurred;
    U32 Dma1ErrorOccurred;
    RA_CDB_START_XFER_FLAG StartXferFlag;
    RA_DMA_CDB_STATUS   CdbStatus[MAX_NUMBER_OF_CDB];
    U32 ProcessedCdbCount;
} RA_DMA_COMMAND_PROCESS, * PTR_RA_DMA_COMMAND_PROCESS;

/* This structure is used for setting of the arbitration mode */
typedef struct _RA_DMA_ARBITRATION_MODE
{
    U8 ArbitrationMode;
    U8 DmaNum;
} RA_DMA_ARBITRATION_MODE, * PTR_RA_DMA_ARBITRATION_MODE;

/* This structure is used by the RAID initialization API. It will only work
 * for Little endian
 */
typedef struct _RA_COMPLETION_QUEUE_CONIFG
{
    U32 Size                 : 2;
    U32 FullDetectionEnabled : 1;
    U32 InterruptCoalescing  : 1;
    U32 InterruptTimer       : 8;
    U32 InterruptCount       : 8;
    U32 Reserved             : 12;
} RA_COMPLETION_QUEUE_CONIFG, * PTR_RA_COMPLETION_QUEUE_CONIFG;

/* This structure is used by the RAID initialization API. It will only work
 * for Little endian
 */
typedef struct _RA_DMA_ENGINE_CONIFG
{
    U32 DMAEnable                   : 1;
    U32 LinkedListArbitrationMode   : 2;
    U32 DataRequestThreshold        : 1;
    U32 InvertCRC                   : 1;
    U32 ParityCheckingEnable        : 1;
    U32 ScatterGatherPrefetchEnable : 1;
    U32 Reserved                    : 1;
    RA_COMPLETION_QUEUE_CONIFG  CompletionQueue;
    U16 AlternateCRCSeed;
} RA_DMA_ENGINE_CONIFG, *PTR_RA_DMA_ENGINE_CONIFG;

/* This structure is used by the RAID initialization API */
typedef struct _RA_CONIFGURATION
{
    RA_DMA_ENGINE_CONIFG DmaEngine[MAX_RACC_DMA];
} RA_CONIFGURATION, * PTR_RA_CONIFGURATION;

extern RAID_ACCELERATOR_BUFFER gRaidAcceleratorBuff;

extern RA_STATUS raidLibiSetDmaArbitrationMode(
                          PTR_RA_DMA_ARBITRATION_MODE PtrDMAArbitrationMode);

extern RA_STATUS raidLibiInit(PTR_RA_CONIFGURATION PtrRaidConfig);

extern RA_STATUS raidLibiSetUpTransfer(U32 LinkListNodeAddr, U8 DmaNum,
                                U8 LinkListType);

extern RA_STATUS raidLibiStartCommandProcessing(
                             PTR_RA_DMA_COMMAND_PROCESS PtrDmaCommandProcess,
                             RA_CDB_START_XFER_FLAG NewCdbFlag, U32 TimeOut);

extern RA_STATUS raidLibiCompletionQueue(U8 DmaNum,
                             PTR_RA_DMA_COMMAND_PROCESS PtrDmaInterrupt);

extern RA_STATUS raidLibiDisableAndCleanUp(void);

#endif
