/***************************************************************************
 *                                                                         *
 *  Copyright 2011 LSI Corporation.  All rights reserved.                  *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corp. The          *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  onfiCpu.h
 *          Title:  ONFI CPU defines
 *     Programmer:  Thibash Rajamani
 *  Creation Date:  June 7, 2011
 *
 *
 * Description
 * -----------
 *   Contains defines for the PPC476FP processor.
 *
 *-------------------------------------------------------------------------
 */

#ifndef ONFI_CPU_H_SOURCE
#define ONFI_CPU_H_SOURCE

#include "onfi.h"

#if __ghs__
/*
 * Store a value in a Device Control Register (DCR).
 *   MoveToDCR(<reg_VALUE>,<CONSTANT_DCRADDR>)
 *   MoveToDCR(<CONSTANT_VALUE>,<reg_DCRADDR>)
 *   MoveToDCR(<CONSTANT_VALUE>,<CONSTANT_DCRADDR>)
 *   MoveToDCR(<reg_VALUE>,<reg_DCRADDR>)
 *
 * NOTE: The move to SPRG0 is a workaround for PPC476FP FDK1 errata #24
 */
asm void MoveToDCR(Value, DCRAddress)
{
%reg Value %con DCRAddress
    lis     r4, %hi(DCRAddress)
    ori     r4, r4, %lo(DCRAddress)
    mtdcrx  r4, Value

%con Value %reg DCRAddress
    lis     r3, %hi(Value)
    ori     r3, r3, %lo(Value)
    mtdcrx  DCRAddress, r3

%con Value %con DCRAddress
    lis     r3, %hi(Value)
    ori     r3, r3, %lo(Value)
    lis     r4, %hi(DCRAddress)
    ori     r4, r4, %lo(DCRAddress)
    mtdcrx  r4, r3

%reg Value %reg DCRAddress
    mtdcrx  DCRAddress, Value

%error
}

/*
 * Get the value of a DCR.
 * myU32 = MoveFromDCR(<CONSTANT_DCRADDR>)
 * myU32 = MoveFromDCR(<reg_DCRADDR>)
 *
 * NOTE: The move to SPRG0 is a workaround for PPC476FP FDK1 errata #24
 */
asm U32 MoveFromDCR(DCRAddress)
{
%   con DCRAddress
    lis     r4, %hi(DCRAddress)
    ori     r4, r4, %lo(DCRAddress)
    mfdcrx  r3, r4

%reg DCRAddress
    mfdcrx  r3, DCRAddress

%error
}

/****************************************************************************
 *
 *   L3 Cache Controller
 *
 ****************************************************************************/

#define L3_CACHE_CONTROLLER_DCR_ADDR            (0x00011000)
#define L3_CACHE_CONTROLLER_DCR_INDIRECT_ADDR   (L3_CACHE_CONTROLLER_DCR_ADDR + 0)
#define L3_CACHE_CONTROLLER_DCR_INDIRECT_DATA   (L3_CACHE_CONTROLLER_DCR_ADDR + 4)
#define L3_CACHE_CONTROLLER_FLUSH_REG_INDEX     (0x00000004) /* L3 Flush */

#define L3_GLOBAL_CONFIG1_REG_INDEX             (0x00000014) /* L3CNFG1 */

/* Bit position of flags in L3 Flush register */
#define L3CC_FLUSH_INIT_BIT     (1)
#define L3CC_FLUSH_DONE_BIT     (0)

/* L3 Global Config1 Register */
typedef struct _L3_GLOBAL_CONFIG1_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32 Reserved0to13                : 14;    /*  0:13 */
            U32 ErrorBufferLinger            :  1;    /*    14 */
            U32 TrueL3CQuiesce               :  1;    /*    15 */
            U32 Reserved16to26               : 11;    /* 26:16 */
            U32 L3Mode                       :  1;    /*    27 */
            U32 Reserved28to30               :  3;    /* 28:30 */
            U32 PlbOrder                     :  1;    /*    31 */
#else  // if defined( _LITTLE_ENDIAN_ )
            U32 PlbOrder                     :  1;    /*    31 */
            U32 Reserved28to30               :  3;    /* 28:30 */
            U32 L3Mode                       :  1;    /*    27 */
            U32 Reserved16to26               : 11;    /* 26:16 */
            U32 TrueL3CQuiesce               :  1;    /*    15 */
            U32 ErrorBufferLinger            :  1;    /*    14 */
            U32 Reserved0to13                : 14;    /*  0:13 */
#endif
        } bits;
        U32 Word;
    } u;
} L3_GLOBAL_CONFIG1_REG, *PTR_L3_GLOBAL_CONFIG1_REG;

INLINE U32 MoveFromL3CacheDCR(U32 Index)
{
    MoveToDCR(Index, L3_CACHE_CONTROLLER_DCR_INDIRECT_ADDR);
    return (MoveFromDCR(L3_CACHE_CONTROLLER_DCR_INDIRECT_DATA));
}

INLINE void MoveToL3CacheDCR(U32 Value, U32 Index)
{
    MoveToDCR(Index, L3_CACHE_CONTROLLER_DCR_INDIRECT_ADDR);
    MoveToDCR(Value, L3_CACHE_CONTROLLER_DCR_INDIRECT_DATA);
}

INLINE void L3_ConfigureCacheAsMemory(BOOL ConfigAsOCM)
{
    L3_GLOBAL_CONFIG1_REG value;
    value.u.Word = MoveFromL3CacheDCR(L3_GLOBAL_CONFIG1_REG_INDEX);

    /* set L3Mode bit : 0 for Cache mode, 1 for OCM Mode */
    value.u.bits.L3Mode = ConfigAsOCM;
    MoveToL3CacheDCR(value.u.Word, L3_GLOBAL_CONFIG1_REG_INDEX);
}

/* getRegVal - Extract a value from a register using a bitmask. Creates the shift on its own.
 *
 * operation - (mask & -mask) gets the least signifcant bit from the mask. When compiled with a
 *             constant the compiler breaks it down to a shift operation(no divide or multiply)
 *             5-6 instructions for a get/set...same as conventional (inw(addr) & mask)>>shift
 *
 * WARNING   - mask must have *only* consecutive 1's
 */
INLINE U32 getRegVal(U32 addr, U32 mask)
{
    U32 value = ((MoveFromDCR(addr) & mask) / (mask & -mask));
    ONFI_REG_DEBUG_PRINT3("READ (getRegVal) addr=0x%x mask=0x%x value=0x%x\n", addr, mask, (value * (mask & -mask)));
    return value;
}

/* setRegVal - Perform a read modify write of a register using a bitmask and creating the shift
 *             operation (mask & -mask) -- See above. Same as conventional outw(addr, inw(addr) & ~x | y<<shift)
 *
 * WARNING   - mask must have *only* consecutive 1's
 */
INLINE void setRegVal(U32 addr, U32 mask, U32 value)
{
    ONFI_REG_DEBUG_PRINT3("WRITE (setRegVal) addr=0x%x mask=0x%x value=0x%x\n", addr, mask, (value * (mask & -mask)));
    MoveToDCR((MoveFromDCR(addr) & ~mask) | (value * (mask & -mask)), addr);
}

/*
 * Force all previous instructions, load, and stores to complete before
 * proceeding.  This instruction is a context synchronizing instruction.
 * MemorySync()
 */
#define MemorySync()  __MSYNC()

asm void __MSYNC( void )
{
    msync
}

/*
 * isync provides an ordering function for the effects of all instructions
 * executed by the processor. Executing isync insures that all instructions
 * preceding the isync instruction execute before isync completes, except
 * that storage accesses caused by those instructions need not have
 * completed. Furthermore, all instructions preceding the isync are
 * guaranteed to be unaffected by any context changes initiated by
 * instructions after the isync.
 */
#define InstructionSync()  __ISYNC()

asm void __ISYNC( void )
{
    isync
}

#define PPC476_CACHE_LINE_SIZE (128)

asm void FlushDataCacheBlock(EA,Index)
{
%   reg Index, EA;
    dcbf EA, Index  /* Data Cache Block Flush instruction - use dcbf here to clear the valid bit */
}

asm void InvalidateDataCacheBlock(EA,Index)
{
%   reg Index, EA;
    dcbi EA, Index
}
#endif /* __ghs__ */

#endif /* ONFI_CPU_H_SOURCE */
