/*******************************************************************************

NAME            ddr3Iop.c
VERSION         %version: 6 %
UPDATE DATE     %date_modified: Tue Nov  5 10:02:52 2013 %
PROGRAMMER      %created_by:    weberc %

        Copyright 2013 NetApp, Inc. All rights reserved.

*******************************************************************************/
/***************************************************************************
 *                                                                         *
 *  Copyright 2012 LSI Corporation.  All rights reserved.                  *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Logic.  The        *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Logic Corporation.                              *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  ddr3Iop.c
 *          Title:  DDR3 IOP functions
 *     Programmer:  Nilesh Wilanker
 *     Modified  :  Jignesh Moradiya
 *  Creation Date:  July 4, 2011
 *
 *  Date      Who   Description
 *  --------  ---   --------------------------------------------------------
 *  07/04/11  NW    Initial version.
 *  08/09/11  NW    Changes based on the review comments. Common functions
                    are shifted to the iopCommon.c file.
 *  08/23/11  NW    Added function iopMemset() to initialize DDR3.
                    Function iopiDelayUSec() shifted to common file.
 *  10/05/11  NM    Porting this to invader
 *  05/20/12  JM    Alignment changes.
 *  05/31/12  JM    Added Function iopDDRSize() for secondary u-boot.
 *
 * Description
 * ------------
 * This file contains the implementation of IOP functions needed by DDR3
 * library.
 *
 *-------------------------------------------------------------------------
 */
#include <basetype.h>
#include <invader.h>
#include <ddr3Init.h>
#include <iopCommon.h>
#include <ubootLib.h>
#include <racc.h>
#include <minidma.h>

#define IFAULT_IOP_BOOTL_INVALID_MEM_CFG_8  (0xB) //  DDR DIMM POST failure
#define IFAULT_IOP_BOOTL_INVALID_MEM_CFG_9  (0xC) //  DIMM Zero Failure

#define SASHW_DCR_CCR                       (0x0000E000)            // Clock, configuration, and reset module

#define SASHW_DCR_CCR_MODSPCRST1_ADDR       (SASHW_DCR_CCR + 0x0D)  // Module Specific Resets 1
#define CCR_MODSPCRST1_DMA_SHELL            (0x01000000)
#define CCR_MODSPCRST1_DMA_PLB              (0x02000000)

#define SAS3HW_RAID_ACC_CFG_RA_ERPN         (0x010)
#define SAS3HW_RAID_ACC_CFG_RA_BASE         (0xC20D0000)
#define SAS3HW_RAID_ACC_CFG_EA_BASE         (0x90000000)

#define CONFIG_SYS_FPGA_BASE                (0xF4000000)
#define FPGA_MISC_CTRL_3_ADDR               (CONFIG_SYS_FPGA_BASE + 0x06)
#define FPGA_MISC_INFO_1_ADDR               (CONFIG_SYS_FPGA_BASE + 0x24)

#define IS_POWERUP()    ( (*(U8*)(FPGA_MISC_INFO_1_ADDR)) & 0x01 )

/*****************************************************************************
 * *   PPC4XX Timer Registers
 * *****************************************************************************/
#define PPC4XX_TIMER_TICKS_PER_10USEC       (10)   /* 1.00 MHz for Silicon */
#define PPC4XX_TIMER_TICKS_PER_10USEC       (10)   /* 1.00 MHz for Silicon */

#define TBL_SPRN                            (268)   // Time Base Register Lower user (RO)

static inline unsigned int ReadTBL()
{
    unsigned int ret=0;

    asm volatile ("mfspr %0,%1" : : "r"(ret), "r"(TBL_SPRN));

    return ret;
}

/* PPC4XX_GET_TICKS should return the number of microseconds */
#define PPC4XX_GET_TICKS() ReadTBL()

/*
 *  * PPC4XX_GET_USEC_TICKS adds the number of ticks associated with AdditionalUSEC
 *   * to the current tick count
 *    */
#if PPC4XX_TIMER_TICKS_PER_10USEC == 10
#define PPC4XX_ADD_USEC_TO_TICKS(AdditionalUSEC)    \
            (PPC4XX_GET_TICKS() + (AdditionalUSEC))
#else
#define PPC4XX_ADD_USEC_TO_TICKS(AdditionalUSEC)    \
            (PPC4XX_GET_TICKS() + (((AdditionalUSEC)*PPC4XX_TIMER_TICKS_PER_10USEC)/10))
#endif

/*
 *  * PPC4XX_CHECK_TICKS checks to see if we've reached the appropriate number of microseconds
 *   * This checks the MSB of a subtraction between the two time values.
 *    * if it is zero then we have not reached the point in time.
 *     * if it is non-zero (one) then we have reached or passed the point in time.
 *      * NOTE: This works with rollover.
 *       */
#define PPC4XX_CHECK_TICKS(NextStamp) \
            ( ~(PPC4XX_GET_TICKS() - (NextStamp)) >> 31 )

static inline void MoveToDCR(unsigned int Value, unsigned int DCRAddress)
{
        asm volatile ("mtdcrx  %0,%1" : : "r"(DCRAddress), "r"(Value));
}

static inline unsigned int MoveFromDCR(unsigned int DCRAddress)
{
        unsigned int ret=0;

       asm volatile ("mfdcrx  %0,%1" : "=r"(ret) : "r"(DCRAddress));

       return ret;
}

#define ISTATUS_INVALID_PARAMETER   2
#define ISTATUS_SUCCESS             0
#define ISTATUS_FAILURE            -1

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: enableResets()
* SUMMARY: Clear the FPGA reset mask
* SCOPE: global
*
* DESCRIPTION:
* Set bit 5 in FPGA Misc Control 3 register to deassert the FPGA reset mask.
* Until this is called, no resets will work. This prevents the alt from
* resetting us until this function is called, which prevents an alt reset
* from ocurring during sensitive times (DDR training, etc) which can cause
* ECC errors.
*
* A similar implementation exists in the regular uboot code as well - this
* function needs to exist in ubootLib so that if any of the LSI code calls
* iopinitFault() we unmaks resets prior to hanging so the alt can reset us.
*
* ARGUMENTS:
* void
*
* RETURNS:
* void
----------------------------------------------------------------------------*/
void enableResets(void)
{
    U8 val = *((volatile U8 *)(FPGA_MISC_CTRL_3_ADDR));

    /* Force everything to sync up in case reset is already asserted */
    asm volatile ("msync");

    /* Set the bit to deassert reset mask */
    *((volatile U8 *)(FPGA_MISC_CTRL_3_ADDR)) = val | 0x20;

    /* For good measure */
    asm volatile ("msync");
}

void iopiDelayUSec( U32 USec )  // PowerPC VERSION!
{
    U32 NextStamp;

    // Return immediately if the delay value is zero.
    if ( USec == 0 )
    {
        return;
    }
    NextStamp = PPC4XX_ADD_USEC_TO_TICKS(USec);

    while( !PPC4XX_CHECK_TICKS(NextStamp) );
}

//#define MEM_STRUCTURE_ALIGN(Pointer, size)  ( (((U32)(Pointer)) + (size)-1) & ~((size)-1))

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: iopinitGetUSec()
* SUMMARY: Get the current tick value of a processor.
* SCOPE: global.
*
* DESCRIPTION:
* Get the current tick value of a processor.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* void
*
* RETURNS:
* Returns the tick value.
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES:
*
----------------------------------------------------------------------------*/
U32 iopinitGetUSec(void)
{
    return (PPC476_GET_TICKS()/TICKS_TO_USEC);
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: iopinitDiagConsolePrint0()
* SUMMARY: Prints the string with no variable arguments.
* SCOPE: global.
*
* DESCRIPTION:
* Prints the string with no variable arguments. This function is called by
* DDR3 library.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* FmtStr : Formatted string to be printed.
*
* RETURNS:
* void
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES: This is the iop function called by DDR3 library.
*
----------------------------------------------------------------------------*/
void iopinitDiagConsolePrint0(char *FmtStr)
{
    iopiDiagConsolePrint0(FmtStr);
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: iopinitDiagConsolePrint1()
* SUMMARY: Prints the string with single variable argument
* SCOPE: global.
*
* DESCRIPTION:
* Prints the string with single variable argument. This function is called by
* DDR3 library.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* FmtStr : Formatted string to be printed.
* Arg1   : First variable argument.
*
* RETURNS:
* void
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES: This is the iop function called by DDR3 library.
*
----------------------------------------------------------------------------*/
void iopinitDiagConsolePrint1(char *FmtStr, U32 Arg1)
{
    iopiDiagConsolePrint1(FmtStr, Arg1);
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: iopinitDiagConsolePrint2()
* SUMMARY: Prints the formatted string with 2 variable arguments.
* SCOPE: global.
*
* DESCRIPTION:
* Prints the formatted string with 2 variable arguments. This function is
* called by DDR3 library.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* FmtStr : Formatted string to be printed.
* Arg1   : First  variable argument.
* Arg2   : Second variable argument.
*
* RETURNS:
* void
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES: This is the iop function called by DDR3 library.
*
----------------------------------------------------------------------------*/
void iopinitDiagConsolePrint2(char *FmtStr, U32 Arg1, U32 Arg2)
{
    iopiDiagConsolePrint2(FmtStr, Arg1, Arg2);
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: iopinitDiagConsolePrint3()
* SUMMARY: Prints the formatted string with 3 variable arguments.
* SCOPE: global.
*
* DESCRIPTION:
* Prints the formatted string with 3 variable arguments. This function is
* called by DDR3 library.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* FmtStr : Formatted string to be printed.
* Arg1   : First  variable argument.
* Arg2   : Second variable argument.
* Arg3   : Third  variable argument.
*
* RETURNS:
* void
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES: This is the iop function called by DDR3 library.
*
----------------------------------------------------------------------------*/
void iopinitDiagConsolePrint3(char *FmtStr, U32 Arg1, U32 Arg2, U32 Arg3)
{
    iopiDiagConsolePrint3(FmtStr, Arg1, Arg2, Arg3);
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: iopinitDiagConsolePrint4()
* SUMMARY: Prints the formatted string with 4 variable arguments.
* SCOPE: global.
*
* DESCRIPTION:
* Prints the formatted string with 4 variable arguments. This function is
* called by DDR3 library.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* FmtStr : Formatted string to be printed.
* Arg1   : First  variable argument.
* Arg2   : Second variable argument.
* Arg3   : Third  variable argument.
* Arg4   : Fourth variable argument.
*
* RETURNS:
* void
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES: This is the iop function called by DDR3 library.
*
----------------------------------------------------------------------------*/
void iopinitDiagConsolePrint4(char * FmtStr, U32 Arg1, U32 Arg2, U32 Arg3,
                                U32 Arg4)
{
    iopiDiagConsolePrint4(FmtStr, Arg1, Arg2, Arg3, Arg4);
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: iopinitMemSet()
* SUMMARY: Initialize the memory space with given value.
* SCOPE: global.
*
* DESCRIPTION:
* Initialize the memory space with given value. This is the iop function needed
* DDR3 library.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* PtrAddr : Pointer to the memory to be initialized.
* Value   : Value with the memory to be initialized.
* Len     : Length of memory to be memset'ed.
*
* RETURNS:
* void
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES: This is the iop function called by DDR3 library.
*
----------------------------------------------------------------------------*/
void * iopinitMemSet(void *PtrAddr, U32 Value, U32 Len)
{
    PS8 xs = (PS8)PtrAddr;

    while (Len--)
    {
        *xs++ = Value;
    }
    return PtrAddr;
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: iopinitFault()
* SUMMARY: Hangs the board.
* SCOPE: global.
*
* DESCRIPTION:
* This function is called by initDdrController() whenever fault condition
* arises and it can no longer proceed. This function will hangs the board
* and request the user to reset the board.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* FaultCode : Fault code passed by DDR3 controller.
*
* RETURNS:
* void
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES: This is the iop function called by DDR3 library.
*
----------------------------------------------------------------------------*/
void iopinitFault(U32 FaultCode)
{
    iopiDiagConsolePrint1("iopFault Code : 0x%x\n", FaultCode);
    iopiDiagConsolePrint0("Please RESET the board.\n");

    /* Something went wrong, allow alt to reset us */
    enableResets();

    /* Hang */
    for(;;);
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: iopinitToUncacheable()
* SUMMARY: Shell routine for the DDR library
* SCOPE: global.
*
* DESCRIPTION:
* This routine returns the input parameter.
* It is a shell routine for the DDR3 library.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* PtrAddr: void pointer to return.
*
* RETURNS:
* void
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES: This is the iop function called by minidma library.
*
----------------------------------------------------------------------------*/
void * iopinitToUncacheable(void *PtrAddr)
{
    return PtrAddr;
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: ddr3cpu_memorySync()
* SUMMARY: Synchronizes the execution.
* SCOPE: global.
*
* DESCRIPTION:
* This function ensures that all the previuos instructions have been completed.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* void
*
* RETURNS:
* void
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES: This is the iop function called by minidma library.
*
----------------------------------------------------------------------------*/
void ddr3cpu_memorySync(void)
{
    asm volatile ("msync");
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: mssResetErrataFix()
* SUMMARY: Sets bits in a special DCR to work around PPC476 chip errata.
* SCOPE: global.
*
* DESCRIPTION:
* See LSI Invader errata 2013-BE750. The errata can result in resets getting
* escalated to the Memory Sub-System (MSS) when they should not be, causing
* DDR to fail to enter self-refresh. This function enables the "pseudo cache
* offload" workaround for the above errata. The bits below set up the MSS to
* start an ONFI cache offload when SYS_RST is detected. When the cache offload
* process begins, the MSS is isolated from the escalated internal reset. Since
* we do not actually set up the rest of the ONFI offload procedure, the MSS
* detects there is no offload to perform and puts the memory into self-refresh
* (while being isolated from the reset). Once the memory is in self-refresh the
* MSS then requires a SYS_POR input to completely reset. Our FPGA asserts the
* SYS_RST and SYS_POR signals as required by the MSS.
*
* ARGUMENTS:
* void
*
* RETURNS:
* void
*
----------------------------------------------------------------------------*/
void mssResetErrataFix(void)
{
    U32 val;

    val = mfdcr(0xE03F);
    val |= 0x82;
    mtdcr(0xE03F, val);
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: iopDDRInit()
* SUMMARY: Initializes the DDR3.
* SCOPE: global.
*
* DESCRIPTION:
* This function initializes the DDR3. Firstly it initializes the miniDMA for
* burst reads and writes and then calls the  initDDRController() to initialize
* the DDR3. initDDRController() will use the miniDMA for burst memcopy.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* argc: number of variable arguments
* argv: pointer to variable arguments
*
* RETURNS:
* 0            : Failure
* Size of DDR3 : Success
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES:
*
----------------------------------------------------------------------------*/
U32 iopDDRInit( int argc, char * argv[])
{
    DDRINIT_MEM_INFO    memInfo;
    U32                 memSize = 0;
    U32                 memDiv;
    U32                 flags = 0;
    U32                 ret;
    U32                 l3Config;
    U32                 clkFreq;
    U32                 nRanks=0;
    U32                 ddrMemSz=0;
    U32                 l3RegionCfgCached =0;
    U32                 l3RegionCfgUnCached =0;
	

    PTR_DDRI_DDR3_SPD_DATA    ptrSpd;

    if( DDR3_INIT != (*(int *)argv[0]) )
    {
        /* Invalid function name passed */
        return 0;
    }

    ptrSpd = (PTR_DDRI_DDR3_SPD_DATA)argv[1];

    /* L3 cache settings : As per DDR3 lib recommendation , we need the cached DDR3 memory as only L3 cached. */
    /* Please note that L3 MEM regions 0 and 1 are cached and 2 and 3 are uncached. */
    /* Using L3 MEM Region 0 and 2 for first rank and region 1 and 3 for second rank.  */

    l3RegionCfgCached = (DDR3_PHY_ADDR_ALIAS_BASE >> 6)+L3_MEM_REGION_SIZE_64MB;
    l3RegionCfgUnCached = (DDR3_PHY_ADDR_ALIAS_BASE_NON_CACHED >> 6)+ L3_MEM_REGION_SIZE_64MB;

    iopiDiagConsolePrint2("First Rank M0 %x  M2 %x \n", l3RegionCfgCached,l3RegionCfgUnCached);
	
    /* First Rank cached DDR3 region */
    mtdcr(L3_INDEX_REG,L3_MEM0_REGION_REG);
    mtdcr(L3_DATA_REG,l3RegionCfgCached);
    /* First Rank non cached DDR3 Region */
    mtdcr(L3_INDEX_REG,L3_MEM2_REGION_REG);
    mtdcr(L3_DATA_REG,l3RegionCfgUnCached);

    mtdcr(L3_INDEX_REG,L3_CONFIG_REG);
    l3Config = mfdcr(L3_DATA_REG);

    l3Config &= 0xFFFFFFEF;

    mtdcr(L3_INDEX_REG,L3_CONFIG_REG);
    mtdcr(L3_DATA_REG,l3Config);

    /* Get MemDiv from DCR reg */
    memDiv  = mfdcr(CCR_DCR_MEMDIV);
    clkFreq = 6667 * ((((memDiv & 0xfc0) >> 6)/2) + 1);

    if((memDiv & 0x80000) == 0)
    {
        clkFreq = clkFreq*2;
    }

    clkFreq  = clkFreq/100;

    nRanks   = ddriSpdGetNumbRanks(ptrSpd);

    ddrMemSz = ddriSpdComputeDimmSizeMb(ddriSpdGetSdramCapacity(ptrSpd), ddriSpdGetBusWidth(ptrSpd), ddriSpdGetSdramDevWidth(ptrSpd), nRanks);

    memInfo.ptrCalibMem                    = (U8 *)(CONFIG_DDR3_BASE0);        /* EA of DDR3 */
    memInfo.ptrCalibMemNoCache             = (U8 *)DDR3_CACHE_INHIBITED_MAP;       /* Cache Inhibited DDR3 Map */
    memInfo.ptrPatBuf                      = (U8 *)(CONFIG_DDR3_PAT_BUF);      /* Not used anywhere by the program */
    memInfo.ptrCmpBuf                      = (U8 *)(CONFIG_DDR3_CMP_BUF);      /* Not used anywhere by the program */
    memInfo.MemCtlRegs                     = NULL;
    memInfo.MemDcrRegs                     = CONFIG_DDR3_DCR_BASE;
    memInfo.RttNom                         = DDRI_RTTNOM_60_OHM;
    memInfo.RttWr                          = DDRI_RTTWR_DISABLED;
    memInfo.DimmOutputDriveStrength        = DDRI_ODS_40_OHM;
    memInfo.DebugForceSiliconRev           = DDRI_SILICON_NO_REV_SPECIFIED;
    memInfo.ptrTrainingData                = (U32 *)(CONFIG_DDR3_TRAINING_DATA);   /* Not used anywhere by the program */
    memInfo.ptrCalibMemSecondRank          = (U8 *)0x00000000;
    memInfo.ptrCalibMemSecondRankNoCache   = (U8 *)0x00000000;

    if (nRanks == 2)
    {
        l3RegionCfgCached =0;
        l3RegionCfgUnCached = 0;
        /* 2GB DIMMs - 2nd rank starts at 1 GB */
        if(ddrMemSz == (1024 * 2))
        {
            memInfo.ptrCalibMemSecondRank         = (U8 *)DDR3_VIRT_ADDR_1GB_2ND_RANK_CACHED;
            memInfo.ptrCalibMemSecondRankNoCache  = (U8 *)DDR3_VIRT_ADDR_1GB_2ND_RANK_NON_CACHED;
            
            l3RegionCfgCached = (DDR3_PHY_ADDR_ALIAS_BASE_1GB >> 6)+L3_MEM_REGION_SIZE_64MB;
            l3RegionCfgUnCached = (DDR3_PHY_ADDR_ALIAS_BASE_1GB_NON_CACHED >> 6)+ L3_MEM_REGION_SIZE_64MB;

        }
        /* 4GB DIMMs - 2nd rank starts at 2 GB */
        else if(ddrMemSz == (1024 * 4))
        {
            memInfo.ptrCalibMemSecondRank         = (U8 *)DDR3_VIRT_ADDR_2GB_2ND_RANK_CACHED;
            memInfo.ptrCalibMemSecondRankNoCache  = (U8 *)DDR3_VIRT_ADDR_2GB_2ND_RANK_NON_CACHED;

            l3RegionCfgCached = (DDR3_PHY_ADDR_ALIAS_BASE_2GB >> 6)+L3_MEM_REGION_SIZE_64MB;
            l3RegionCfgUnCached = (DDR3_PHY_ADDR_ALIAS_BASE_2GB_NON_CACHED >> 6)+ L3_MEM_REGION_SIZE_64MB;			
			
        }
        /* 8GB DIMMs - 2nd rank starts at 4 GB */		
        else if(ddrMemSz == (1024 * 8))
        {
            memInfo.ptrCalibMemSecondRank         = (U8 *)DDR3_VIRT_ADDR_4GB_2ND_RANK_CACHED;
            memInfo.ptrCalibMemSecondRankNoCache  = (U8 *)DDR3_VIRT_ADDR_4GB_2ND_RANK_NON_CACHED;
			
            l3RegionCfgCached = (DDR3_PHY_ADDR_ALIAS_BASE_4GB >> 6)+L3_MEM_REGION_SIZE_64MB;
            l3RegionCfgUnCached = (DDR3_PHY_ADDR_ALIAS_BASE_4GB_NON_CACHED >> 6)+ L3_MEM_REGION_SIZE_64MB;			
			
        }

        /* 
         * Push training memory 2K into the buffer to avoid corrupting any nearby
         * memory during training, per recommendation by LSI.  We only need to
         * do this for the second rank, as addresses for the first are in the middle
         * of processor memory that will be zeroed out anyway.
         */  
        memInfo.ptrCalibMemSecondRank         += (2*1024);
        memInfo.ptrCalibMemSecondRankNoCache  += (2*1024);

	/* 2nd Rank L3 cached region */
        mtdcr(L3_INDEX_REG,L3_MEM1_REGION_REG);
        mtdcr(L3_DATA_REG,l3RegionCfgCached);
	/* 2nd Rank L3 non- cached region */
        mtdcr(L3_INDEX_REG,L3_MEM3_REGION_REG);
        mtdcr(L3_DATA_REG,l3RegionCfgUnCached);	

    }

    /* If powerup, DDR is not in self-refresh, so do not pass this flag.
     * If not powerup, DDR is in self-refresh and we must take it out. */
    if (!IS_POWERUP())
    {
        flags |= DDR_INIT_EXIT_SELF_REFRESH;
    }

    mssResetErrataFix();

    ret = initDDRController((void *)ptrSpd, &memSize, &memInfo, clkFreq, flags);
    if (ret)
    {
        /* Init failed */
        ret = 0;
    }
    else
    {
        ret = memSize;
    }

    /* Restore the original L3 settings */

    l3RegionCfgCached =L3_MEM_REGION_INVALID;
    l3RegionCfgUnCached = 0;
	
    /* No L3 cached regions */
    mtdcr(L3_INDEX_REG,L3_MEM0_REGION_REG);
    mtdcr(L3_DATA_REG,l3RegionCfgCached);

    mtdcr(L3_INDEX_REG,L3_MEM1_REGION_REG);
    mtdcr(L3_DATA_REG,l3RegionCfgCached);

    /* Entire DDR3 space is  L3 inhibited */
    l3RegionCfgUnCached = (DDR3_PHY_ADDR_ALIAS_BASE >> 6) + L3_MEM_REGION_SIZE_16GB;
    mtdcr(L3_INDEX_REG,L3_MEM2_REGION_REG);
    mtdcr(L3_DATA_REG,l3RegionCfgUnCached);

    l3RegionCfgUnCached = L3_MEM_REGION_STARTADDRESS_DDR3_NORMAL + L3_MEM_REGION_SIZE_2GB;
    mtdcr(L3_INDEX_REG,L3_MEM3_REGION_REG);
    mtdcr(L3_DATA_REG,l3RegionCfgUnCached);
	
    ddr3cpu_memorySync();
    ddriHandleBogusMcError();

    return (ret);
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: iopDDRSize ()
* SUMMARY: Return the Size of DDR3.
* SCOPE: global.
*
* DESCRIPTION:
* This function returns the Size of DDR3.
* This Function is called by secondary u-boot.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* argc: number of variable arguments
* argv: pointer to variable arguments
*
* RETURNS:
* 0            : Failure
* Size of DDR3 : Success
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES:
*
----------------------------------------------------------------------------*/
U32 iopDDRSize (int argc, char * argv[])
{
    PTR_DDRI_DDR3_SPD_DATA ptrSpd;
    U32                    ddrMemSz;

    if(DDR3_SIZE != (*(int *)argv[0]))
    {
        /* Invalid function name passed */
        return 0;
    }

    /* Initialise SPD Pointer */
    ptrSpd   = (PTR_DDRI_DDR3_SPD_DATA)argv[1];

    ddrMemSz = ddriSpdComputeDimmSizeMb(ddriSpdGetSdramCapacity(ptrSpd), ddriSpdGetBusWidth(ptrSpd), ddriSpdGetSdramDevWidth(ptrSpd), ddriSpdGetNumbRanks(ptrSpd));

    return (ddrMemSz);
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: ddrPostUseDma()
* SUMMARY: Initailize the DDR after the training.
* SCOPE: global.
*
* DESCRIPTION:
* This function initailize the DDR using the miniDMA library.The miniDMA
* library is designed to provide a fast and efficient means for clearing
* large amounts of DDR SDRAM, initializing the memorys ECC,* and testing
* the memory. It uses the RAID accelerator DMA engines in LSI controllers
* to perform these tasks.
*
* Pre-conditions:
* DDR3 training shoulb complete successfully.
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* argc: number of variable arguments
* argv: pointer to variable arguments
*
* RETURNS:
* int - status 0 for success and != 0 for failure
*
* ERROR:
*   1.Timeout - when the DMA operation is not completed within a given time
*   2.Error interrupts - DMA engine is halted because of the error interrupts.
*   3.Invalid DDR size - If the DDR size is passed as 0
*
* RESTRICTIONS:
*   1.CDB and completion que locations must be UNCACHED and LITTLE ENDIAN.
*
* NOTES:
*
----------------------------------------------------------------------------*/
U32 ddrPostUseDma(int argc , char * argv[])
{
    DDR3_MINI_DMA_INFO      miniDmaInfo;
    MINI_DMA_XFER           dmaXfer;
    U32                     timeout;
    U32                     modSpcReset = 0;
    U32  *ddr3Addr = (U32 *)argv[0];
    U32  *ddr3Size = (U32 *)argv[1];
    U64 dest;
    U32 count;

    dest.High = *ddr3Addr;
    dest.Low  = *(ddr3Addr+1);
    count     = *ddr3Size;

    /* Dont proceed if the size is 0*/
    if(count == 0)
    {
        iopiDiagConsolePrint0( "Size of the DDR is zero..Returning \n");
        return 1;
    }

    /*Reset Katana*/
    modSpcReset  = MoveFromDCR( SASHW_DCR_CCR_MODSPCRST1_ADDR );
    modSpcReset |= (CCR_MODSPCRST1_DMA_SHELL| CCR_MODSPCRST1_DMA_PLB);
    MoveToDCR( modSpcReset, SASHW_DCR_CCR_MODSPCRST1_ADDR );
    iopinitDelayUSec(1000);
    modSpcReset &= ~(CCR_MODSPCRST1_DMA_SHELL| CCR_MODSPCRST1_DMA_PLB);
    MoveToDCR( modSpcReset, SASHW_DCR_CCR_MODSPCRST1_ADDR );
    iopinitDelayUSec(1000);

    /*Fill out miniDMA structure for locations used*/
    miniDmaInfo.ptrCDBs         = (RACC_CDB_UNION *)(U8 *)MEM_STRUCTURE_ALIGN((U32)CONFIG_SYS_ISRAM_BASE3, 32);
    miniDmaInfo.ptrCompQ        = (COMPLETION_QUEUE_ENTRY *)(&miniDmaInfo.ptrCDBs[MINI_DMA_MAX_CDBS]);
    miniDmaInfo.ptrRaidAccRegs  = (PTR_RACC_DMA_REGS)(SAS3HW_RAID_ACC_CFG_EA_BASE + RACC_TOP_PLB_CONFIG_OFFSET);

    /* Initailize the miniDMA lib */
    miniDmaInit(&miniDmaInfo);

    /* Timeout needs to vary based upon size as DDR init could take a LOT of time.
     * Based upon 400 MTS DDR2 this appears to be about 1250 microseconds per 1MB
     */
    timeout = (count/(1024*1024)+1)*1250;
    /*sets up the transfer structure.*/
    miniDmaOpenXfer(&dmaXfer);
    /* Write zeros and check */
    miniDmaAddFill(&dmaXfer,MAKE64(dest.High,dest.Low),count,0);
    miniDmaAddCheck(&dmaXfer,MAKE64(dest.High,dest.Low),count,0);
    /* wait for it to complete */
    if (miniDmaIssueXferAndWait(&dmaXfer,timeout) != MINI_DMA_STATUS_NORMAL)
    {
       iopinitFault(IFAULT_IOP_BOOTL_INVALID_MEM_CFG_9);
    }
    /* Clean up structures and hardware */
    miniDmaDisableAndCleanUp();

    return dmaXfer.status;
}
