/***************************************************************************
 *                                                                         *
 *  Copyright 2012 LSI Corporation.  All rights reserved.                  *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation.  The  *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  secUbootLib.c
 *          Title:  Secondary U-boot functions for dual core.
 *     Programmer:  Jignesh Moradiya
 *  Creation Date:  May 20,2012
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   --------------------------------------------------------
 *  05/20/12  JM    Initial version.
 *
 * Description
 * ------------
 * This file contains the implementation of secondary u-boot functions for
 * dual core library.
 *
 *-------------------------------------------------------------------------
 */

/* Dualcore header files */
#include <basetype.h>
#include <ubootLib.h>     /* U-Boot - Standalone Header            */
#include <secUbootLib.h>  /* Secondary U-Boot  - Standalone Header */
#include <dcDB.h>         /* Dual Core - Doorbell Header           */
#include <dcQP.h>         /* Dual Core - Queue Pointer Header      */

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: secUbootLib()
* SUMMARY: Secondary u-boot library's entry point.
* SCOPE: global.
*
* DESCRIPTION:
* Secondary u-boot library's entry point. It will parse the function name and
* its arguments and invoke the corresponding dual core API.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* argc: number of variable arguments
* argv: pointer to variable arguments
*
* RETURNS:
* DC_SUCCESS and other related to functions: Success
* DC_FAIL and other related to functions : Failure
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES:
*
----------------------------------------------------------------------------*/
U32 secUbootLib( int argc, char * argv[] )
{
    U32 ret                       = DC_SUCCESS;
    PTR_DC_FUNCTION ptrDcFunction = (PTR_DC_FUNCTION)argv[0];

    switch ( ptrDcFunction->FunctionName )
    {
        case DC_SET_ENABLE_DB_INT:
            if ( ptrDcFunction->Argc == 0 )
            {
                ret = dcSetEnableDBInt(DC_DB_ENABLE_INT_DEFAULT);
            }
            else if ( ptrDcFunction->Argc == 1 )
            {
                /* Argument: Enable Interrupt */
                ret = dcSetEnableDBInt(*(PU32)ptrDcFunction->PtrArgv[0]);
            }
            break;

        case DC_GET_DB_STATUS:
            if ( ptrDcFunction->Argc == 1 )
            {
                /* Argument: Pointer of  DC_PMU_DB_REG */
                ret = dcGetDBStatus(
                        (PTR_DC_PMU_DB_REG)ptrDcFunction->PtrArgv[0]);
            }
            break;

        case DC_SET_DB_INT:
            if ( ptrDcFunction->Argc == 0 )
            {
                /* DC_DB_ASSERTION_DEFAULT = 0x00000001 */
                ret = dcSetDBInt(DC_DB_ASSERTION_DEFAULT);
            }
            else if ( ptrDcFunction->Argc == 1 )
            {
                /* Argument: DB Interrupt value */
                ret = dcSetDBInt(*(PU32)ptrDcFunction->PtrArgv[0]);
            }
            break;

        case DC_WRITE_QP:
            if ( ptrDcFunction->Argc == 2 )
            {
                /* Argument: Queue number, Message for write operation */
                ret = dcWriteQP(*(PU32)ptrDcFunction->PtrArgv[0],
                                (char *)ptrDcFunction->PtrArgv[1]);
            }
            break;

        case DC_RESET_TXQ:
            if ( ptrDcFunction->Argc == 1 )
            {
                /* Argument: Queue number */
                ret = dcResetTxQ(*(PU32)ptrDcFunction->PtrArgv[0]);
            }
            break;

        case DC_RESET_RXQ:
            if ( ptrDcFunction->Argc == 1 )
            {
                /* Argument: Queue number */
                ret = dcResetRxQ(*(PU32)ptrDcFunction->PtrArgv[0]);
            }
            break;

        case DC_SET_COAL_INT_CNT:
            if ( ptrDcFunction->Argc == 2 )
            {
                /* Arguments: Queue number,
                 *            Coalescing Interrupt counter value
                 */
                ret = dcSetCoalIntCnt(*(PU32)ptrDcFunction->PtrArgv[0],
                                      *(PU32)ptrDcFunction->PtrArgv[1]);
            }
            break;

        case DC_SET_COAL_INT_TIMER:
            if ( ptrDcFunction->Argc == 3 )
            {
                 /* Arguments: Queue number,
                  *            coalescing interrupt timer value,
                  *            Tick
                  */
                ret = dcSetCoalIntTimer(*(PU32)ptrDcFunction->PtrArgv[0],
                                        *(PU32)ptrDcFunction->PtrArgv[1],
                                        *(PU32)ptrDcFunction->PtrArgv[2]);
            }
            break;

        case DC_INIT_PMU:
            if ( ptrDcFunction->Argc == 0 )
            {
                dcInitDB();
                ret = DC_SUCCESS;
            }
            else if ( ptrDcFunction->Argc == 3 )
            {
               /* Arguments: Queue number , Address, Size */
                dcInitQP(*(PU32)ptrDcFunction->PtrArgv[0],
                         *(PU32)ptrDcFunction->PtrArgv[1],
                         *(PU32)ptrDcFunction->PtrArgv[2]);
                ret = DC_SUCCESS;
            }
            break;

        case SEC_PMU_INT_HANDLER:
            if ( ptrDcFunction->Argc == 2 )
            {
                if ( (int)ptrDcFunction->PtrArgv[0] == GET_DC_DB_STATUS )
                {
                    /* Argument:  Pointer of DC_PMU_DB_REG */
                    ret = GET_DC_DB_STS(
                            (PTR_DC_PMU_DB_REG)ptrDcFunction->PtrArgv[1]);
                }
                else if ( (int)ptrDcFunction->PtrArgv[0] == CLEAR_DC_DB_INT )
                {
                    U32 curInt;
                    /* Argument:  Pointer of DC_PMU_DB_REG */
                    curInt = GET_DC_DBPRIORITY(
                                (PTR_DC_PMU_DB_REG)ptrDcFunction->PtrArgv[1]);
                    ret    = dcClearDBInt(curInt);
                }
            }
            else if ( ptrDcFunction->Argc == 3 )
            {
                PTR_DC_PMU_DB_REG ptrIntSts =
                        (PTR_DC_PMU_DB_REG)ptrDcFunction->PtrArgv[0];
                int *ptrRdBufSize = (int *)ptrDcFunction->PtrArgv[2];

                /* Interrupt generated due to RxQ0 not empty*/
                if (GET_DC_RXQ0_NOT_EMPTY(ptrIntSts))
                {
                    ret = dcReadQP(0, (char *)ptrDcFunction->PtrArgv[1],
                             ptrRdBufSize);
                }
                /* Interrupt generated due to RxQ1 not empty*/
                else if (GET_DC_RXQ1_NOT_EMPTY(ptrIntSts))
                {
                    ret = dcReadQP(1, (char *)ptrDcFunction->PtrArgv[1],
                             ptrRdBufSize);
                }
                /* Interrupt generated due to RxQ2 not empty*/
                else if (GET_DC_RXQ2_NOT_EMPTY(ptrIntSts))
                {
                    ret = dcReadQP(2, (char *)ptrDcFunction->PtrArgv[1],
                             ptrRdBufSize);
                }
                /* Interrupt generated due to RxQ3 not empty*/
                else if (GET_DC_RXQ3_NOT_EMPTY(ptrIntSts))
                {
                    ret = dcReadQP(3, (char *)ptrDcFunction->PtrArgv[1],
                             ptrRdBufSize);
                }
                else
                {
                    ret = DC_FAIL;
                }
            }
            break;

        default:
            /* Invalid function name passed */
            return DC_FAIL;
    }

    return (ret);
}
